@@ -36,12 +36,15 @@ import org.apache.cassandra.sidecar.common.request.ConnectedClientStatsRequest;
 import org.apache.cassandra.sidecar.common.request.CreateSnapshotRequest;
 import org.apache.cassandra.sidecar.common.request.GossipHealthRequest;
 import org.apache.cassandra.sidecar.common.request.GossipInfoRequest;
+import org.apache.cassandra.sidecar.common.request.GossipUpdateRequest;
 import org.apache.cassandra.sidecar.common.request.ImportSSTableRequest;
 import org.apache.cassandra.sidecar.common.request.ListOperationalJobsRequest;
 import org.apache.cassandra.sidecar.common.request.ListSnapshotFilesRequest;
+import org.apache.cassandra.sidecar.common.request.NativeUpdateRequest;
 import org.apache.cassandra.sidecar.common.request.NodeDecommissionRequest;
 import org.apache.cassandra.sidecar.common.request.NodeSettingsRequest;
 import org.apache.cassandra.sidecar.common.request.OperationalJobRequest;
+import org.apache.cassandra.sidecar.common.request.ReportSchemaRequest;
 import org.apache.cassandra.sidecar.common.request.Request;
 import org.apache.cassandra.sidecar.common.request.RingRequest;
 import org.apache.cassandra.sidecar.common.request.SSTableComponentRequest;
@@ -53,8 +56,10 @@ import org.apache.cassandra.sidecar.common.request.TimeSkewRequest;
 import org.apache.cassandra.sidecar.common.request.TokenRangeReplicasRequest;
 import org.apache.cassandra.sidecar.common.request.UploadSSTableRequest;
 import org.apache.cassandra.sidecar.common.request.data.Digest;
+import org.apache.cassandra.sidecar.common.request.data.NodeCommandRequestPayload;
 import org.apache.cassandra.sidecar.common.response.ListSnapshotFilesResponse;
 import org.apache.cassandra.sidecar.common.utils.HttpRange;
+import org.jetbrains.annotations.NotNull;
 import org.jetbrains.annotations.Nullable;
 
 /**
@@ -83,7 +88,7 @@ public class RequestContext
     protected static final NodeDecommissionRequest NODE_DECOMMISSION_REQUEST = new NodeDecommissionRequest();
 
     protected static final StreamStatsRequest STREAM_STATS_REQUEST = new StreamStatsRequest();
-    protected static final RetryPolicy DEFAULT_RETRY_POLICY = new NoRetryPolicy();
+    protected static final RetryPolicy DEFAULT_NO_RETRY_POLICY = new NoRetryPolicy();
     protected static final RetryPolicy DEFAULT_EXPONENTIAL_BACKOFF_RETRY_POLICY =
     new ExponentialBackoffRetryPolicy(10, 500L, 60_000L);
 
@@ -139,7 +144,7 @@ public class RequestContext
     {
         private InstanceSelectionPolicy instanceSelectionPolicy;
         private Request request;
-        private RetryPolicy retryPolicy = DEFAULT_RETRY_POLICY;
+        private RetryPolicy retryPolicy = DEFAULT_NO_RETRY_POLICY;
         private final Map<String, String> customHeaders;
 
         public Builder()
@@ -497,6 +502,16 @@ public class RequestContext
             return request(new UploadSSTableRequest(keyspace, tableName, uploadId, component, digest, filename));
         }
 
+        /**
+         * Sets the {@code request} to be a new instance of a {@link ReportSchemaRequest}
+         *
+         * @return this {@link Builder} for method chaining
+         */
+        public Builder reportSchemaRequest()
+        {
+            return request(new ReportSchemaRequest());
+        }
+
         /**
          * Sets the {@code request} to be a {@link ConnectedClientStatsRequest} and returns a reference to this Builder
          * enabling method chaining.
@@ -552,6 +567,30 @@ public class RequestContext
             return request(NODE_DECOMMISSION_REQUEST);
         }
 
+        /**
+         * Sets the {@code request} to be a {@link GossipUpdateRequest} for the
+         * given {@link NodeCommandRequestPayload.State state}, and returns a reference to this Builder enabling method chaining.
+         *
+         * @param state  the desired state for gossip
+         * @return a reference to this Builder
+         */
+        public Builder nodeGossipUpdateRequest(@NotNull NodeCommandRequestPayload.State state)
+        {
+            return request(new GossipUpdateRequest(state));
+        }
+
+        /**
+         * Sets the {@code request} to be a {@link NativeUpdateRequest} for the
+         * given {@link NodeCommandRequestPayload.State state}, and returns a reference to this Builder enabling method chaining.
+         *
+         * @param state  the desired state for native transport
+         * @return a reference to this Builder
+         */
+        public Builder nodeNativeUpdateRequest(@NotNull NodeCommandRequestPayload.State state)
+        {
+            return request(new NativeUpdateRequest(state));
+        }
+
         /**
          * Sets the {@code request} to be a {@link StreamStatsRequest} and returns a reference to this Builder
          * enabling method chaining.
@@ -563,6 +602,16 @@ public class RequestContext
             return request(STREAM_STATS_REQUEST);
         }
 
+        /**
+         * Sets the {@code retryPolicy} to be an instance of {@link NoRetryPolicy}
+         *
+         * @return this {@link Builder} for method chaining
+         */
+        public Builder noRetryPolicy()
+        {
+            return retryPolicy(DEFAULT_NO_RETRY_POLICY);
+        }
+
         /**
          * Sets the {@code retryPolicy} to be an
          * {@link org.apache.cassandra.sidecar.client.retry.ExponentialBackoffRetryPolicy} configured with
