@@ -20,6 +20,7 @@
 package org.apache.cassandra.sidecar.client;
 
 import java.util.List;
+import java.util.Map;
 import java.util.Objects;
 import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
@@ -30,29 +31,40 @@ import org.slf4j.LoggerFactory;
 import io.netty.handler.codec.http.HttpResponseStatus;
 import org.apache.cassandra.sidecar.client.retry.CreateRestoreJobRetryPolicy;
 import org.apache.cassandra.sidecar.client.retry.IgnoreConflictRetryPolicy;
+import org.apache.cassandra.sidecar.client.retry.LiveMigrationDownloadRetryPolicy;
 import org.apache.cassandra.sidecar.client.retry.OncePerInstanceRetryPolicy;
 import org.apache.cassandra.sidecar.client.retry.RetryPolicy;
 import org.apache.cassandra.sidecar.client.retry.RunnableOnStatusCodeRetryPolicy;
 import org.apache.cassandra.sidecar.client.selection.InstanceSelectionPolicy;
 import org.apache.cassandra.sidecar.client.selection.RandomInstanceSelectionPolicy;
 import org.apache.cassandra.sidecar.common.request.AbortRestoreJobRequest;
+import org.apache.cassandra.sidecar.common.request.AllServicesConfigRequest;
 import org.apache.cassandra.sidecar.common.request.CreateRestoreJobRequest;
 import org.apache.cassandra.sidecar.common.request.CreateRestoreJobSliceRequest;
+import org.apache.cassandra.sidecar.common.request.DeleteServiceConfigRequest;
 import org.apache.cassandra.sidecar.common.request.ImportSSTableRequest;
 import org.apache.cassandra.sidecar.common.request.ListCdcSegmentsRequest;
+import org.apache.cassandra.sidecar.common.request.LiveMigrationListInstanceFilesRequest;
 import org.apache.cassandra.sidecar.common.request.RestoreJobProgressRequest;
 import org.apache.cassandra.sidecar.common.request.RestoreJobSummaryRequest;
+import org.apache.cassandra.sidecar.common.request.Service;
 import org.apache.cassandra.sidecar.common.request.StreamCdcSegmentRequest;
+import org.apache.cassandra.sidecar.common.request.StreamFileRequest;
 import org.apache.cassandra.sidecar.common.request.UpdateRestoreJobRequest;
+import org.apache.cassandra.sidecar.common.request.UpdateServiceConfigRequest;
 import org.apache.cassandra.sidecar.common.request.data.AbortRestoreJobRequestPayload;
+import org.apache.cassandra.sidecar.common.request.data.AllServicesConfigPayload;
 import org.apache.cassandra.sidecar.common.request.data.CreateRestoreJobRequestPayload;
 import org.apache.cassandra.sidecar.common.request.data.CreateSliceRequestPayload;
 import org.apache.cassandra.sidecar.common.request.data.Digest;
+import org.apache.cassandra.sidecar.common.request.data.NodeCommandRequestPayload;
 import org.apache.cassandra.sidecar.common.request.data.RestoreJobProgressRequestParams;
+import org.apache.cassandra.sidecar.common.request.data.UpdateCdcServiceConfigPayload;
 import org.apache.cassandra.sidecar.common.request.data.UpdateRestoreJobRequestPayload;
 import org.apache.cassandra.sidecar.common.response.ConnectedClientStatsResponse;
 import org.apache.cassandra.sidecar.common.response.GossipInfoResponse;
 import org.apache.cassandra.sidecar.common.response.HealthResponse;
+import org.apache.cassandra.sidecar.common.response.InstanceFilesListResponse;
 import org.apache.cassandra.sidecar.common.response.ListCdcSegmentsResponse;
 import org.apache.cassandra.sidecar.common.response.ListOperationalJobsResponse;
 import org.apache.cassandra.sidecar.common.response.ListSnapshotFilesResponse;
@@ -114,6 +126,21 @@ public class SidecarClient implements AutoCloseable, SidecarClientBlobRestoreExt
                                             .build());
     }
 
+    /**
+     * Executes the Sidecar health request using the configured selection policy and with no retries
+     *
+     * @param instance the instance where the request will be executed
+     * @return a completable future of the Sidecar health response
+     */
+    public CompletableFuture<HealthResponse> sidecarHealth(SidecarInstance instance)
+    {
+        return executor.executeRequestAsync(requestBuilder()
+                                            .singleInstanceSelectionPolicy(instance)
+                                            .retryPolicy(oncePerInstanceRetryPolicy)
+                                            .sidecarHealthRequest()
+                                            .build());
+    }
+
     /**
      * Executes the Cassandra health request using the configured selection policy and with no retries
      *
@@ -518,9 +545,9 @@ public class SidecarClient implements AutoCloseable, SidecarClientBlobRestoreExt
     public CompletableFuture<ListCdcSegmentsResponse> listCdcSegments(SidecarInstance sidecarInstance)
     {
         return executor.executeRequestAsync(requestBuilder()
-                       .singleInstanceSelectionPolicy(sidecarInstance)
-                       .request(new ListCdcSegmentsRequest())
-                       .build());
+                                            .singleInstanceSelectionPolicy(sidecarInstance)
+                                            .request(new ListCdcSegmentsRequest())
+                                            .build());
     }
 
     /**
@@ -540,9 +567,65 @@ public class SidecarClient implements AutoCloseable, SidecarClientBlobRestoreExt
                                   StreamConsumer streamConsumer)
     {
         executor.streamRequest(requestBuilder()
-                .singleInstanceSelectionPolicy(sidecarInstance)
-                .request(new StreamCdcSegmentRequest(segment, range))
-                .build(), streamConsumer);
+                               .singleInstanceSelectionPolicy(sidecarInstance)
+                               .request(new StreamCdcSegmentRequest(segment, range))
+                               .build(), streamConsumer);
+    }
+
+    /**
+     * Sends a request to trigger an immediate, synchronous schema
+     * conversion and report on the specified instance of the Sidecar
+     * regardless of the periodic task schedule or status
+     *
+     * @param instance the {@link SidecarInstance} to receive the request
+     * @return a {@link CompletableFuture} for the request
+     */
+    public <T> CompletableFuture<T> reportSchema(SidecarInstance instance)
+    {
+        return executor.executeRequestAsync(requestBuilder()
+                                            .singleInstanceSelectionPolicy(instance)
+                                            .reportSchemaRequest()
+                                            .noRetryPolicy()  // {@link NoRetryPolicy} is the preferred behavior here
+                                            .build());
+    }
+
+    /**
+     * Get configs for all the services in the "configs" table inside sidecar's internal
+     * keyspace
+     *
+     * @return List of services and their corresponding configs
+     */
+    public CompletableFuture<AllServicesConfigPayload> allServicesConfig()
+    {
+        return executor.executeRequestAsync(requestBuilder()
+                                            .request(new AllServicesConfigRequest())
+                                            .build());
+    }
+
+    /**
+     * Update config for a given service in "configs" table in internal sidecar keyspace
+     *
+     * @param service service for which the configs are being updated
+     * @param config the updated config
+     * @return updated config
+     */
+    public CompletableFuture<UpdateCdcServiceConfigPayload> updateCdcServiceConfig(Service service, Map<String, String> config)
+    {
+        return executor.executeRequestAsync(requestBuilder()
+                                            .request(new UpdateServiceConfigRequest(service, new UpdateCdcServiceConfigPayload(config)))
+                                            .build());
+    }
+
+    /**
+     * Deletes configs for a given service in "configs" table in internal sidecar keyspace
+     *
+     * @param service service for which the configs are being deleted
+     */
+    public CompletableFuture<Void> deleteCdcServiceConfig(Service service)
+    {
+        return executor.executeRequestAsync(requestBuilder()
+                                            .request(new DeleteServiceConfigRequest(service))
+                                            .build());
     }
 
     /**
@@ -724,6 +807,47 @@ public class SidecarClient implements AutoCloseable, SidecarClientBlobRestoreExt
                                             .build());
     }
 
+    /**
+     * Sends a request to start or stop Cassandra gossiping on the provided instance.
+     * <p>
+     * This operation asynchronously triggers start or stop of gossip via Cassandra's JMX interface.
+     * The request body must contain a JSON payload with a "state" field, which can be either "start" or "stop".
+     * On success, the server responds with HTTP 200 OK and payload {@code {"status":"OK"}}.
+     * </p>
+     *
+     * @param instance the instance where the request will be executed
+     * @param state the desired gossip state: {@link NodeCommandRequestPayload.State#START} or {@link NodeCommandRequestPayload.State#STOP}
+     * @return a CompletableFuture representing the completion of the operation
+     */
+    public CompletableFuture<HealthResponse> nodeUpdateGossip(SidecarInstance instance, NodeCommandRequestPayload.State state)
+    {
+        return executor.executeRequestAsync(requestBuilder()
+                                            .singleInstanceSelectionPolicy(instance)
+                                            .nodeGossipUpdateRequest(state)
+                                            .build());
+    }
+
+    /**
+     * Sends a request to start or stop Cassandra native transport on the provided instance.
+     * <p>
+     * This operation asynchronously triggers start or stop of native transport via Cassandra's JMX interface.
+     * The request body must contain a JSON payload with a "state" field, which can be either "start" or "stop".
+     * On success, the server responds with HTTP 200 OK and payload {@code {"status":"OK"}}.
+     * </p>
+     *
+     * @param instance the instance where the request will be executed
+     * @param state the desired native transport state: {@link NodeCommandRequestPayload.State#START} or {@link NodeCommandRequestPayload.State#STOP}
+     * @return a CompletableFuture representing the completion of the operation
+     */
+    public CompletableFuture<HealthResponse> nodeUpdateNative(SidecarInstance instance, NodeCommandRequestPayload.State state)
+    {
+        return executor.executeRequestAsync(requestBuilder()
+                                            .singleInstanceSelectionPolicy(instance)
+                                            .nodeNativeUpdateRequest(state)
+                                            .build());
+    }
+
+
     /**
      * Returns a copy of the request builder with the default parameters configured for the client.
      *
@@ -770,6 +894,37 @@ public class SidecarClient implements AutoCloseable, SidecarClientBlobRestoreExt
         return executor.executeRequestAsync(context);
     }
 
+    /**
+     * Gets list of files available at a Cassandra Instance using Sidecar during Live Migration.
+     *
+     * @param instance Sidecar instance where the request will be executed
+     * @return a CompletableFuture representing the completion of the operation
+     */
+    public CompletableFuture<InstanceFilesListResponse> liveMigrationListInstanceFilesAsync(SidecarInstance instance)
+    {
+        return executor.executeRequestAsync(requestBuilder().singleInstanceSelectionPolicy(instance)
+                                                            .request(new LiveMigrationListInstanceFilesRequest())
+                                                            .build());
+    }
+
+
+    /**
+     * Downloads requested file {@code reqPath} to the destination file provided with param {@code targetFile}
+     * during Live Migration.
+     *
+     * @param instance       the instance where the request will be executed
+     * @param reqPath        instance file path to copy
+     * @param targetFilePath destination file path to save file coming from remote
+     * @return a CompletableFuture representing the completion of the operation
+     */
+    public CompletableFuture<Void> liveMigrationStreamFileAsync(SidecarInstance instance, String reqPath, String targetFilePath)
+    {
+        return executor.executeRequestAsync(requestBuilder().singleInstanceSelectionPolicy(instance)
+                                                            .request(new StreamFileRequest(reqPath, targetFilePath))
+                                                            .retryPolicy(new LiveMigrationDownloadRetryPolicy(defaultRetryPolicy, targetFilePath))
+                                                            .build());
+    }
+
     /**
      * {@inheritDoc}
      */
