@@ -29,7 +29,9 @@ import java.nio.file.StandardCopyOption;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.UUID;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CountDownLatch;
@@ -66,12 +68,18 @@ import org.apache.cassandra.sidecar.common.data.RestoreJobSecrets;
 import org.apache.cassandra.sidecar.common.request.ImportSSTableRequest;
 import org.apache.cassandra.sidecar.common.request.NodeSettingsRequest;
 import org.apache.cassandra.sidecar.common.request.Request;
+import org.apache.cassandra.sidecar.common.request.Service;
+import org.apache.cassandra.sidecar.common.request.data.AllServicesConfigPayload;
 import org.apache.cassandra.sidecar.common.request.data.CreateRestoreJobRequestPayload;
 import org.apache.cassandra.sidecar.common.request.data.MD5Digest;
+import org.apache.cassandra.sidecar.common.request.data.NodeCommandRequestPayload;
+import org.apache.cassandra.sidecar.common.request.data.UpdateCdcServiceConfigPayload;
 import org.apache.cassandra.sidecar.common.request.data.XXHash32Digest;
 import org.apache.cassandra.sidecar.common.response.ConnectedClientStatsResponse;
 import org.apache.cassandra.sidecar.common.response.GossipInfoResponse;
 import org.apache.cassandra.sidecar.common.response.HealthResponse;
+import org.apache.cassandra.sidecar.common.response.InstanceFileInfo;
+import org.apache.cassandra.sidecar.common.response.InstanceFilesListResponse;
 import org.apache.cassandra.sidecar.common.response.ListCdcSegmentsResponse;
 import org.apache.cassandra.sidecar.common.response.ListOperationalJobsResponse;
 import org.apache.cassandra.sidecar.common.response.ListSnapshotFilesResponse;
@@ -99,14 +107,18 @@ import static io.netty.handler.codec.http.HttpResponseStatus.OK;
 import static io.netty.handler.codec.http.HttpResponseStatus.PARTIAL_CONTENT;
 import static org.apache.cassandra.sidecar.common.ApiEndpointsV1.JOB_ID_PATH_PARAM;
 import static org.apache.cassandra.sidecar.common.ApiEndpointsV1.KEYSPACE_PATH_PARAM;
+import static org.apache.cassandra.sidecar.common.ApiEndpointsV1.LIVE_MIGRATION_FILES_API;
 import static org.apache.cassandra.sidecar.common.ApiEndpointsV1.OPERATIONAL_JOB_ID_PATH_PARAM;
 import static org.apache.cassandra.sidecar.common.ApiEndpointsV1.TABLE_PATH_PARAM;
 import static org.apache.cassandra.sidecar.common.http.SidecarHttpHeaderNames.CONTENT_XXHASH32;
 import static org.apache.cassandra.sidecar.common.http.SidecarHttpHeaderNames.CONTENT_XXHASH32_SEED;
+import static org.apache.cassandra.sidecar.common.response.InstanceFileInfo.FileType.DIRECTORY;
+import static org.apache.cassandra.sidecar.common.response.InstanceFileInfo.FileType.FILE;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.assertj.core.api.Assertions.assertThatException;
 import static org.assertj.core.api.Assertions.assertThatExceptionOfType;
 import static org.assertj.core.api.Assertions.assertThatIllegalArgumentException;
+import static org.assertj.core.api.Assertions.assertThatNoException;
 import static org.assertj.core.api.Assertions.assertThatThrownBy;
 import static org.assertj.core.api.Assertions.fail;
 
@@ -333,8 +345,8 @@ abstract class SidecarClientTest
         assertThat(entry.fqdn()).isEqualTo("local");
         assertThat(entry.hostId()).isEqualTo("000");
 
-        validateResponseServed(ApiEndpointsV1.RING_ROUTE_PER_KEYSPACE.replaceAll(KEYSPACE_PATH_PARAM,
-                                                                                 "cycling"));
+        validateResponseServed(ApiEndpointsV1.RING_WITH_KEYSPACE_ROUTE.replaceAll(KEYSPACE_PATH_PARAM,
+                                                                                  "cycling"));
     }
 
     @Test
@@ -391,7 +403,7 @@ abstract class SidecarClientTest
         assertThat(gossipInfo.releaseVersion()).isEqualTo("4.0.7");
         assertThat(gossipInfo.sstableVersions()).isEqualTo(Collections.singletonList("big-nb"));
 
-        validateResponseServed(ApiEndpointsV1.GOSSIP_INFO_ROUTE);
+        validateResponseServed(ApiEndpointsV1.GOSSIP_ROUTE);
     }
 
     @Test
@@ -654,6 +666,7 @@ abstract class SidecarClientTest
         client.createSnapshot(sidecarInstance, "cycling", "cyclist_name", "2023.04.11")
               .get(30, TimeUnit.SECONDS);
 
+        assertThat(mockWebServer.getRequestCount()).isEqualTo(1);
         assertThat(mockWebServer.getRequestCount()).isEqualTo(1);
         RecordedRequest request = mockWebServer.takeRequest();
         assertThat(request.getPath()).isEqualTo(ApiEndpointsV1.SNAPSHOTS_ROUTE
@@ -1565,7 +1578,12 @@ abstract class SidecarClientTest
         long expectedTotalSize = 2048;
         long expectedSnapshotSize = 100;
 
-        TableStatsResponse tableStatsResponse = new TableStatsResponse(testKeyspace, testTable, expectedSstables, expectedSize, expectedTotalSize, expectedSnapshotSize);
+        TableStatsResponse tableStatsResponse = new TableStatsResponse(testKeyspace,
+                                                                       testTable,
+                                                                       expectedSstables,
+                                                                       expectedSize,
+                                                                       expectedTotalSize,
+                                                                       expectedSnapshotSize);
         ObjectMapper mapper = new ObjectMapper();
         MockResponse response = new MockResponse().setResponseCode(OK.code())
                                                   .setBody(mapper.writeValueAsString(tableStatsResponse));
@@ -1594,7 +1612,7 @@ abstract class SidecarClientTest
     public void testListCdcSegments() throws ExecutionException, InterruptedException, JsonProcessingException
     {
         List<CdcSegmentInfo> segments = Arrays.asList(new CdcSegmentInfo("commit-log1", 100, 100, true, 1732148713725L),
-                new CdcSegmentInfo("commit-log2", 100, 10, false, 1732148713725L));
+                                                      new CdcSegmentInfo("commit-log2", 100, 10, false, 1732148713725L));
         ListCdcSegmentsResponse listSegmentsResponse = new ListCdcSegmentsResponse("localhost", 9043, segments);
         ObjectMapper mapper = new ObjectMapper();
 
@@ -1660,6 +1678,88 @@ abstract class SidecarClientTest
         assertThat(new String(baos.toByteArray(), StandardCharsets.UTF_8)).isEqualTo("Test Content");
     }
 
+    @Test
+    public void testReportSchemaSuccess()
+    {
+        MockResponse response = new MockResponse().setResponseCode(OK.code())
+                                                  .setBody("{\"status\":\"OK\"}");
+
+        enqueue(response);
+
+        SidecarInstance instance = instances.get(0);
+
+        assertThatNoException().isThrownBy(() -> client.reportSchema(instance).get());
+    }
+
+    @Test
+    public void testReportSchemaFailure()
+    {
+        MockResponse response = new MockResponse().setResponseCode(INTERNAL_SERVER_ERROR.code())
+                                                  .setBody(INTERNAL_SERVER_ERROR.reasonPhrase());
+
+        enqueue(response);
+
+        SidecarInstance instance = instances.get(0);
+
+        assertThatThrownBy(() -> client.reportSchema(instance).get()).isExactlyInstanceOf(ExecutionException.class)
+                                                                     .hasCauseInstanceOf(RetriesExhaustedException.class)
+                                                                     .hasMessageContaining(Integer.toString(INTERNAL_SERVER_ERROR.code()))
+                                                                     .hasMessageContaining(INTERNAL_SERVER_ERROR.reasonPhrase());
+    }
+
+    @Test
+    public void testAllServiceSuccessTests() throws IOException, ExecutionException, InterruptedException
+    {
+        List<AllServicesConfigPayload.Service> services = new ArrayList<>();
+        Map<String, String> kafkaConfigs = new HashMap<>();
+        kafkaConfigs.put("k1", "v1");
+        kafkaConfigs.put("k2", "v2");
+        Map<String, String> cdcConfigs = new HashMap<>();
+        cdcConfigs.put("k1", "v1");
+        cdcConfigs.put("k2", "v2");
+        services.add(new AllServicesConfigPayload.Service("kafka", kafkaConfigs));
+        services.add(new AllServicesConfigPayload.Service("cdc", cdcConfigs));
+        AllServicesConfigPayload expectedResponse = new AllServicesConfigPayload(services);
+
+        MockResponse response = new MockResponse();
+        response.setResponseCode(200);
+        response.setHeader("content-type", "application/json");
+        ObjectMapper mapper = new ObjectMapper();
+        response.setBody(mapper.writeValueAsString(expectedResponse));
+        enqueue(response);
+        assertThat(client.allServicesConfig().get()).isEqualTo(expectedResponse);
+        validateResponseServed(ApiEndpointsV1.SERVICES_CONFIG_ROUTE,
+                               request -> assertThat(request.getMethod()).isEqualTo("GET"));
+    }
+
+    @Test
+    public void testUpdateConfigSuccessTests() throws IOException, ExecutionException, InterruptedException
+    {
+        Map<String, String> payload = new HashMap<>();
+        payload.put("testKey", "testValue");
+        UpdateCdcServiceConfigPayload putResponse = new UpdateCdcServiceConfigPayload(payload);
+        MockResponse response = new MockResponse();
+        response.setResponseCode(200);
+        response.setHeader("content-type", "application/json");
+        ObjectMapper mapper = new ObjectMapper();
+        response.setBody(mapper.writeValueAsString(putResponse));
+        enqueue(response);
+        assertThat(client.updateCdcServiceConfig(Service.CDC, payload).get()).isEqualTo(putResponse);
+        validateResponseServed(ApiEndpointsV1.SERVICE_CONFIG_ROUTE.replaceAll(ApiEndpointsV1.SERVICE_PARAM, "cdc"),
+                               request -> assertThat(request.getMethod()).isEqualTo("PUT"));
+    }
+
+    @Test
+    public void testDeleteConfigTests() throws ExecutionException, InterruptedException
+    {
+        MockResponse response = new MockResponse();
+        response.setResponseCode(200);
+        enqueue(response);
+        client.deleteCdcServiceConfig(Service.CDC).get();
+        validateResponseServed(ApiEndpointsV1.SERVICE_CONFIG_ROUTE.replaceAll(ApiEndpointsV1.SERVICE_PARAM, "cdc"),
+                               request -> assertThat(request.getMethod()).isEqualTo("DELETE"));
+    }
+
     @Test
     public void testStreamsStats() throws Exception
     {
@@ -1680,6 +1780,120 @@ abstract class SidecarClientTest
         }
     }
 
+    @Test
+    void testNodeUpdateGossip() throws Exception
+    {
+        MockResponse response = new MockResponse().setResponseCode(200).setBody("{\"status\":\"OK\"}");
+        enqueue(response);
+
+        SidecarInstanceImpl sidecarInstance = instances.get(0);
+        HealthResponse result = client.nodeUpdateGossip(sidecarInstance, NodeCommandRequestPayload.State.STOP).get(30, TimeUnit.SECONDS);
+        assertThat(result).isNotNull();
+        assertThat(result.status()).isEqualToIgnoringCase("OK");
+        assertThat(result.isOk()).isTrue();
+
+        validateResponseServed(ApiEndpointsV1.GOSSIP_ROUTE, request -> {
+            String requestBody = request.getBody().readUtf8();
+            assertThat(requestBody).isEqualTo("{\"state\":\"stop\"}");
+        });
+    }
+
+    @Test
+    void testNodeUpdateNative() throws Exception
+    {
+        MockResponse response = new MockResponse().setResponseCode(200).setBody("{\"status\":\"OK\"}");
+        enqueue(response);
+
+        SidecarInstanceImpl sidecarInstance = instances.get(0);
+        HealthResponse result = client.nodeUpdateNative(sidecarInstance, NodeCommandRequestPayload.State.STOP).get(30, TimeUnit.SECONDS);
+        assertThat(result).isNotNull();
+        assertThat(result.status()).isEqualToIgnoringCase("OK");
+        assertThat(result.isOk()).isTrue();
+
+        validateResponseServed(ApiEndpointsV1.CASSANDRA_NATIVE_ROUTE, request -> {
+            String requestBody = request.getBody().readUtf8();
+            assertThat(requestBody).isEqualTo("{\"state\":\"stop\"}");
+        });
+    }
+
+    @Test
+    void testLiveMigrationListInstanceFiles() throws ExecutionException, InterruptedException, JsonProcessingException
+    {
+        List<InstanceFileInfo> fileInfoList = Arrays.asList(new InstanceFileInfo("commit-log1", 100, FILE, System.currentTimeMillis()),
+                                                            new InstanceFileInfo("commit-log2", 100, DIRECTORY, System.currentTimeMillis()));
+        InstanceFilesListResponse filesListResponse = new InstanceFilesListResponse(fileInfoList);
+        ObjectMapper mapper = new ObjectMapper();
+
+        MockResponse response = new MockResponse();
+        response.setResponseCode(200);
+        response.setHeader("content-type", "application/json");
+        response.setBody(mapper.writeValueAsString(filesListResponse));
+        enqueue(response);
+
+        SidecarInstance instance = instances.get(0);
+        InstanceFilesListResponse result = client.liveMigrationListInstanceFilesAsync(instance).get();
+        assertThat(result).isNotNull();
+        assertThat(result).isEqualTo(filesListResponse);
+        validateResponseServed(ApiEndpointsV1.LIVE_MIGRATION_FILES_API);
+    }
+
+    @Test
+    void testLiveMigrationStreamFileAsync(@TempDir Path tempDirectory) throws ExecutionException, InterruptedException, IOException
+    {
+        String dummyFileContent = "Some dummy file content";
+        MockResponse response = new MockResponse();
+        response.setResponseCode(200);
+        response.setHeader("content-type", "application/text");
+        response.setBody(dummyFileContent);
+        enqueue(response);
+
+        Path filePath = tempDirectory.resolve("test_file.txt");
+        SidecarInstance instance = instances.get(0);
+        String fileUrl = LIVE_MIGRATION_FILES_API + "/data/0/test_file.text";
+
+        client.liveMigrationStreamFileAsync(instance, fileUrl, filePath.toString()).get();
+
+        assertThat(Files.exists(filePath)).isTrue();
+        assertThat(new String(Files.readAllBytes(filePath), StandardCharsets.UTF_8)).isEqualTo(dummyFileContent);
+        validateResponseServed(fileUrl);
+    }
+
+    @Test
+    void testLiveMigrationStreamFileAsyncFileNotFound(@TempDir Path tempDirectory) throws ExecutionException, InterruptedException
+    {
+        MockResponse response = new MockResponse();
+        response.setResponseCode(404);
+        enqueue(response);
+
+        Path filePath = tempDirectory.resolve("test_file.txt");
+        SidecarInstance instance = instances.get(0);
+        String fileUrl = LIVE_MIGRATION_FILES_API + "/data/0/test_file.text";
+
+        client.liveMigrationStreamFileAsync(instance, fileUrl, filePath.toString()).get();
+
+        assertThat(Files.exists(filePath)).isFalse();
+        validateResponseServed(fileUrl);
+    }
+
+    @Test
+    void testLiveMigrationStreamFileAsyncBadRequest(@TempDir Path tempDirectory) throws InterruptedException
+    {
+        MockResponse response = new MockResponse();
+        response.setResponseCode(400);
+        enqueue(response);
+
+        Path filePath = tempDirectory.resolve("test_file.txt");
+        SidecarInstance instance = instances.get(0);
+        String fileUrl = LIVE_MIGRATION_FILES_API + "/data/0/test_file.text";
+
+        CompletableFuture<Void> result = client.liveMigrationStreamFileAsync(instance, fileUrl, filePath.toString());
+
+        assertThatExceptionOfType(ExecutionException.class).isThrownBy(result::get);
+        assertThat(result.isCompletedExceptionally()).isTrue();
+        assertThat(Files.exists(filePath)).isFalse();
+        validateResponseServed(fileUrl);
+    }
+
     private void enqueue(MockResponse response)
     {
         for (MockWebServer server : servers)
