@@ -19,7 +19,6 @@
 
 package org.apache.cassandra.sidecar.testing;
 
-import java.io.IOException;
 import java.net.BindException;
 import java.net.InetSocketAddress;
 import java.net.URI;
@@ -53,6 +52,7 @@ import com.datastax.driver.core.SimpleStatement;
 import com.google.inject.AbstractModule;
 import com.google.inject.Guice;
 import com.google.inject.Injector;
+import com.google.inject.Module;
 import com.google.inject.Provides;
 import com.google.inject.Singleton;
 import com.google.inject.util.Modules;
@@ -90,8 +90,9 @@ import org.apache.cassandra.sidecar.config.yaml.SchemaKeyspaceConfigurationImpl;
 import org.apache.cassandra.sidecar.config.yaml.ServiceConfigurationImpl;
 import org.apache.cassandra.sidecar.config.yaml.SidecarConfigurationImpl;
 import org.apache.cassandra.sidecar.config.yaml.SslConfigurationImpl;
+import org.apache.cassandra.sidecar.coordination.ClusterLease;
 import org.apache.cassandra.sidecar.metrics.instance.InstanceHealthMetrics;
-import org.apache.cassandra.sidecar.server.MainModule;
+import org.apache.cassandra.sidecar.modules.SidecarModules;
 import org.apache.cassandra.sidecar.server.Server;
 import org.apache.cassandra.sidecar.server.SidecarServerEvents;
 import org.apache.cassandra.sidecar.utils.CassandraVersionProvider;
@@ -104,6 +105,7 @@ import org.apache.cassandra.testing.TestVersionSupplier;
 
 import static org.apache.cassandra.sidecar.config.yaml.S3ClientConfigurationImpl.DEFAULT_API_CALL_TIMEOUT;
 import static org.apache.cassandra.sidecar.testing.MtlsTestHelper.CASSANDRA_INTEGRATION_TEST_ENABLE_MTLS;
+import static org.apache.cassandra.testing.utils.IInstanceUtils.tryGetIntConfig;
 import static org.assertj.core.api.Assertions.assertThat;
 
 /**
@@ -153,12 +155,10 @@ public abstract class SharedClusterIntegrationTestBase
 
     protected DnsResolver dnsResolver = new LocalhostResolver();
     protected IClusterExtension<? extends IInstance> cluster;
-    protected Server server;
+    protected ServerWrapper serverWrapper;
     protected TestVersion testVersion;
     protected MtlsTestHelper mtlsTestHelper;
-    private final CountDownLatch sidecarSchemaReadyLatch = new CountDownLatch(1);
     private IsolatedDTestClassLoaderWrapper classLoaderWrapper;
-    private Injector sidecarServerInjector;
 
     static
     {
@@ -321,6 +321,8 @@ public abstract class SharedClusterIntegrationTestBase
 
     /**
      * Override to provide additional options to configure sidecar
+     *
+     * @return function to update {@link SidecarConfigurationImpl.Builder}
      */
     protected Function<SidecarConfigurationImpl.Builder, SidecarConfigurationImpl.Builder> configurationOverrides()
     {
@@ -335,41 +337,61 @@ public abstract class SharedClusterIntegrationTestBase
      */
     protected void startSidecar(ICluster<? extends IInstance> cluster) throws InterruptedException
     {
-        server = startSidecarWithInstances(cluster);
+        serverWrapper = startSidecarWithInstances(cluster);
     }
 
     /**
      * Starts Sidecar configured to run with the provided {@link IInstance}s from the cluster.
      *
      * @param instances the Cassandra instances Sidecar will manage
+     * @return a wrapper with the started server
+     * @throws InterruptedException when the server start operation is interrupted
+     */
+    protected ServerWrapper startSidecarWithInstances(Iterable<? extends IInstance> instances) throws InterruptedException
+    {
+        return startSidecarWithInstances(instances, null);
+    }
+
+    /**
+     * Starts Sidecar configured to run with the provided {@link IInstance}s from the cluster.
+     *
+     * @param instances    the Cassandra instances Sidecar will manage
+     * @param customModule an optional custom module that overrides during injection
      * @return the started server
      * @throws InterruptedException when the server start operation is interrupted
      */
-    protected Server startSidecarWithInstances(Iterable<? extends IInstance> instances) throws InterruptedException
+    protected ServerWrapper startSidecarWithInstances(Iterable<? extends IInstance> instances, AbstractModule customModule) throws InterruptedException
     {
         VertxTestContext context = new VertxTestContext();
-        AbstractModule testModule = new IntegrationTestModule(instances, classLoaderWrapper, mtlsTestHelper,
+        AbstractModule testModule = new IntegrationTestModule(instances, mtlsTestHelper,
                                                               dnsResolver, configurationOverrides());
-        sidecarServerInjector = Guice.createInjector(Modules.override(new MainModule()).with(testModule));
-        Vertx vertx = sidecarServerInjector.getInstance(Vertx.class);
-        vertx.eventBus()
-             .localConsumer(SidecarServerEvents.ON_SIDECAR_SCHEMA_INITIALIZED.address(), msg -> sidecarSchemaReadyLatch.countDown());
-        Server sidecarServer = sidecarServerInjector.getInstance(Server.class);
+        Module module = testModule;
+        if (customModule != null)
+        {
+            module = Modules.override(testModule).with(customModule);
+        }
+        Injector injector = Guice.createInjector(Modules.override(SidecarModules.all()).with(module));
+        Server sidecarServer = injector.getInstance(Server.class);
         sidecarServer.start()
                      .onSuccess(s -> context.completeNow())
                      .onFailure(context::failNow);
 
-        context.awaitCompletion(5, TimeUnit.SECONDS);
-        return sidecarServer;
+        assertThat(context.awaitCompletion(5, TimeUnit.SECONDS)).isTrue();
+        return new ServerWrapper(injector, sidecarServer);
     }
 
     protected void waitForSchemaReady(long timeout, TimeUnit timeUnit)
     {
-        assertThat(sidecarServerInjector)
+        waitForSchemaReady(serverWrapper, timeout, timeUnit);
+    }
+
+    protected void waitForSchemaReady(ServerWrapper serverWrapper, long timeout, TimeUnit timeUnit)
+    {
+        assertThat(serverWrapper)
         .describedAs("Sidecar should be started")
         .isNotNull();
 
-        assertThat(Uninterruptibles.awaitUninterruptibly(sidecarSchemaReadyLatch, timeout, timeUnit))
+        assertThat(Uninterruptibles.awaitUninterruptibly(serverWrapper.sidecarSchemaReadyLatch, timeout, timeUnit))
         .describedAs("Sidecar schema is not initialized after " + timeout + ' ' + timeUnit)
         .isTrue();
     }
@@ -381,12 +403,17 @@ public abstract class SharedClusterIntegrationTestBase
      */
     protected void stopSidecar() throws InterruptedException
     {
-        if (server == null)
+        closeServer(serverWrapper.server);
+    }
+
+    protected void closeServer(Server s) throws InterruptedException
+    {
+        if (s == null)
         {
             return;
         }
         CountDownLatch closeLatch = new CountDownLatch(1);
-        server.close().onSuccess(res -> closeLatch.countDown());
+        s.close().onSuccess(res -> closeLatch.countDown());
         if (closeLatch.await(60, TimeUnit.SECONDS))
         {
             logger.info("Close event received before timeout.");
@@ -511,6 +538,30 @@ public abstract class SharedClusterIntegrationTestBase
         return builder.build();
     }
 
+    /**
+     * Wraps the Sidecar server and keeps a reference to the injector to be able to dynamically retrieve
+     * objects for testing purposes
+     */
+    public static class ServerWrapper
+    {
+        public final Injector injector;
+        public final Server server;
+        public volatile int serverPort;
+        private final CountDownLatch sidecarSchemaReadyLatch = new CountDownLatch(1);
+
+        public ServerWrapper(Injector sidecarServerInjector, Server server)
+        {
+            this.injector = sidecarServerInjector;
+            this.server = server;
+            // Server must have started to retrieve the port
+            this.serverPort = server.actualPort();
+
+            Vertx vertx = sidecarServerInjector.getInstance(Vertx.class);
+            vertx.eventBus().localConsumer(SidecarServerEvents.ON_SIDECAR_SCHEMA_INITIALIZED.address(),
+                                           msg -> sidecarSchemaReadyLatch.countDown());
+        }
+    }
+
     /**
      * Test module that configures the instances based on the cluster instances
      */
@@ -518,20 +569,17 @@ public abstract class SharedClusterIntegrationTestBase
     {
         private static final Logger LOGGER = LoggerFactory.getLogger(IntegrationTestModule.class);
         private final Iterable<? extends IInstance> instances;
-        private final IsolatedDTestClassLoaderWrapper wrapper;
         private final MtlsTestHelper mtlsTestHelper;
         private final DnsResolver dnsResolver;
         private final SidecarVersionProvider sidecarVersionProvider = new SidecarVersionProvider();
         private final Function<SidecarConfigurationImpl.Builder, SidecarConfigurationImpl.Builder> configurationOverrides;
 
         public IntegrationTestModule(Iterable<? extends IInstance> instances,
-                                     IsolatedDTestClassLoaderWrapper wrapper,
                                      MtlsTestHelper mtlsTestHelper,
                                      DnsResolver dnsResolver,
                                      Function<SidecarConfigurationImpl.Builder, SidecarConfigurationImpl.Builder> configurationOverrides)
         {
             this.instances = instances;
-            this.wrapper = wrapper;
             this.mtlsTestHelper = mtlsTestHelper;
             this.configurationOverrides = configurationOverrides;
             this.dnsResolver = dnsResolver;
@@ -558,6 +606,7 @@ public abstract class SharedClusterIntegrationTestBase
         public InstancesMetadata instancesMetadata(Vertx vertx,
                                                    SidecarConfiguration configuration,
                                                    CassandraVersionProvider cassandraVersionProvider,
+                                                   SidecarVersionProvider sidecarVersionProvider,
                                                    CQLSessionProvider cqlSessionProvider,
                                                    DnsResolver dnsResolver)
         {
@@ -570,8 +619,7 @@ public abstract class SharedClusterIntegrationTestBase
                                                                 sidecarVersionProvider.sidecarVersion(),
                                                                 jmxConfiguration,
                                                                 cqlSessionProvider,
-                                                                dnsResolver,
-                                                                wrapper))
+                                                                dnsResolver))
                          .collect(Collectors.toList());
             return new InstancesMetadataImpl(instanceMetadataList, dnsResolver);
         }
@@ -579,6 +627,34 @@ public abstract class SharedClusterIntegrationTestBase
         @Provides
         @Singleton
         public SidecarConfiguration sidecarConfiguration()
+        {
+            return defaultConfigurationBuilder(mtlsTestHelper, configurationOverrides).build();
+        }
+
+        @Provides
+        @Singleton
+        public DnsResolver dnsResolver()
+        {
+            return dnsResolver;
+        }
+
+        @Provides
+        @Singleton
+        public ClusterLease clusterLease()
+        {
+            return new ClusterLease(ClusterLease.Ownership.CLAIMED);
+        }
+
+        private List<InetSocketAddress> buildContactPoints()
+        {
+            return StreamSupport.stream(instances.spliterator(), false)
+                                .map(instance -> new InetSocketAddress(instance.config().broadcastAddress().getAddress(),
+                                                                       tryGetIntConfig(instance, "native_transport_port", 9042)))
+                                .collect(Collectors.toList());
+        }
+
+        public static SidecarConfigurationImpl.Builder defaultConfigurationBuilder(
+        MtlsTestHelper mtlsTestHelper, Function<SidecarConfigurationImpl.Builder, SidecarConfigurationImpl.Builder> configurationOverrides)
         {
             ServiceConfiguration conf = ServiceConfigurationImpl.builder()
                                                                 .host("0.0.0.0") // binds to all interfaces, potential security issue if left running for long
@@ -620,33 +696,19 @@ public abstract class SharedClusterIntegrationTestBase
             S3ClientConfiguration s3ClientConfig = new S3ClientConfigurationImpl("s3-client", 4, SecondBoundConfiguration.parse("60s"),
                                                                                  5242880, DEFAULT_API_CALL_TIMEOUT,
                                                                                  buildTestS3ProxyConfig());
+
             SidecarConfigurationImpl.Builder builder = SidecarConfigurationImpl.builder()
                                                                                .serviceConfiguration(conf)
-                                                                               .sslConfiguration(sslConfiguration)
-                                                                               .s3ClientConfiguration(s3ClientConfig);
+                                                                               .s3ClientConfiguration(s3ClientConfig)
+                                                                               .sslConfiguration(sslConfiguration);
             if (configurationOverrides != null)
             {
                 builder = configurationOverrides.apply(builder);
             }
-            return builder.build();
+            return builder;
         }
 
-        @Provides
-        @Singleton
-        public DnsResolver dnsResolver()
-        {
-            return dnsResolver;
-        }
-
-        private List<InetSocketAddress> buildContactPoints()
-        {
-            return StreamSupport.stream(instances.spliterator(), false)
-                                .map(instance -> new InetSocketAddress(instance.config().broadcastAddress().getAddress(),
-                                                                       tryGetIntConfig(instance.config(), "native_transport_port", 9042)))
-                                .collect(Collectors.toList());
-        }
-
-        private S3ProxyConfiguration buildTestS3ProxyConfig()
+        private static S3ProxyConfiguration buildTestS3ProxyConfig()
         {
             return new S3ProxyConfiguration()
             {
@@ -676,15 +738,17 @@ public abstract class SharedClusterIntegrationTestBase
             };
         }
 
-        static int tryGetIntConfig(IInstanceConfig config, String configName, int defaultValue)
+        public static String cassandraInstanceHostname(IInstance cassandraInstance, DnsResolver dnsResolver)
         {
+            IInstanceConfig config = cassandraInstance.config();
+            String ipAddress = JMXUtil.getJmxHost(config);
             try
             {
-                return config.getInt(configName);
+                return dnsResolver.reverseResolve(ipAddress);
             }
-            catch (NullPointerException npe)
+            catch (UnknownHostException e)
             {
-                return defaultValue;
+                return ipAddress;
             }
         }
 
@@ -694,8 +758,7 @@ public abstract class SharedClusterIntegrationTestBase
                                                       String sidecarVersion,
                                                       JmxConfiguration jmxConfiguration,
                                                       CQLSessionProvider session,
-                                                      DnsResolver dnsResolver,
-                                                      IsolatedDTestClassLoaderWrapper wrapper)
+                                                      DnsResolver dnsResolver)
         {
             IInstanceConfig config = cassandraInstance.config();
             String ipAddress = JMXUtil.getJmxHost(config);
@@ -712,12 +775,12 @@ public abstract class SharedClusterIntegrationTestBase
             String[] dataDirectories = (String[]) config.get("data_file_directories");
             String stagingDir = stagingDir(dataDirectories);
 
-            JmxClient jmxClient = new JmxClientProxy(wrapper,
-                                                     JmxClient.builder()
-                                                              .host(ipAddress)
-                                                              .port(config.jmxPort())
-                                                              .connectionMaxRetries(jmxConfiguration.maxRetries())
-                                                              .connectionRetryDelay(jmxConfiguration.retryDelay()));
+            JmxClient jmxClient = JmxClient.builder()
+                                           .host(ipAddress)
+                                           .port(config.jmxPort())
+                                           .connectionMaxRetries(jmxConfiguration.maxRetries())
+                                           .connectionRetryDelay(jmxConfiguration.retryDelay())
+                                           .build();
             MetricRegistry metricRegistry = new MetricRegistry();
             CassandraAdapterDelegate delegate = new CassandraAdapterDelegate(vertx,
                                                                              config.num(),
@@ -754,39 +817,4 @@ public abstract class SharedClusterIntegrationTestBase
             return stagingPath.toFile().getAbsolutePath();
         }
     }
-
-    protected JmxClient wrapJmxClient(JmxClient.Builder builder)
-    {
-        return new JmxClientProxy(classLoaderWrapper, builder);
-    }
-
-    /**
-     * Runs JMXClient calls with the dtest classloader
-     */
-    static class JmxClientProxy extends JmxClient
-    {
-        private final IsolatedDTestClassLoaderWrapper wrapper;
-
-        protected JmxClientProxy(IsolatedDTestClassLoaderWrapper wrapper, Builder builder)
-        {
-            super(Objects.requireNonNull(builder, "builder must be provided"));
-            this.wrapper = wrapper;
-        }
-
-        /**
-         * Connects to JMX by running inside the wrapped classloader. The connection to JMX must be established in the
-         * dtest classloader to be able to communicate to Cassandra's JMX.
-         *
-         * @param currentAttempt the current attempt to connect
-         * @throws IOException if the underlying operation throws an IOException
-         */
-        @Override
-        protected void connectInternal(int currentAttempt) throws IOException
-        {
-            wrapper.executeExceptionableActionOnDTestClassLoader(() -> {
-                super.connectInternal(currentAttempt);
-                return null;
-            });
-        }
-    }
 }
