@@ -64,6 +64,7 @@ import org.jetbrains.annotations.NotNull;
 import static org.apache.cassandra.sidecar.adapters.base.jmx.EndpointSnitchJmxOperations.ENDPOINT_SNITCH_INFO_OBJ_NAME;
 import static org.apache.cassandra.sidecar.adapters.base.jmx.StorageJmxOperations.STORAGE_SERVICE_OBJ_NAME;
 import static org.apache.cassandra.sidecar.exceptions.CassandraUnavailableException.Service.CQL_AND_JMX;
+import static org.apache.cassandra.sidecar.exceptions.CassandraUnavailableException.Service.JMX;
 import static org.apache.cassandra.sidecar.server.SidecarServerEvents.ON_CASSANDRA_CQL_DISCONNECTED;
 import static org.apache.cassandra.sidecar.server.SidecarServerEvents.ON_CASSANDRA_CQL_READY;
 import static org.apache.cassandra.sidecar.server.SidecarServerEvents.ON_CASSANDRA_JMX_DISCONNECTED;
@@ -114,6 +115,7 @@ public class CassandraAdapterDelegate implements ICassandraAdapter, Host.StateLi
      * @param sidecarVersion      the version of the Sidecar from the current binary
      * @param host                the Cassandra instance's hostname or ip address as a string
      * @param port                the Cassandra instance's port number
+     * @param healthMetrics       health metrics
      */
     public CassandraAdapterDelegate(Vertx vertx,
                                     int cassandraInstanceId,
@@ -135,7 +137,7 @@ public class CassandraAdapterDelegate implements ICassandraAdapter, Host.StateLi
         this.cqlSessionProvider = session;
         this.jmxClient = jmxClient;
         this.healthMetrics = healthMetrics;
-        notificationListener = initializeJmxListener();
+        this.notificationListener = initializeJmxListener();
     }
 
     // TODO: re-organize the methods in the class to group the public/protected/private methods together
@@ -369,6 +371,10 @@ public class CassandraAdapterDelegate implements ICassandraAdapter, Host.StateLi
     @NotNull
     public NodeSettings nodeSettings() throws CassandraUnavailableException
     {
+        if (nodeSettingsFromJmx == null)
+        {
+            throw new CassandraUnavailableException(JMX, "NodeSettings unavailable");
+        }
         return nodeSettingsFromJmx;
     }
 
