@@ -26,5 +26,5 @@ public interface CoordinationConfiguration
     /**
      * @return configuration parameters for the cluster lease claim task
      */
-    PeriodicTaskConfiguration clusterLeaseClaimConfiguration();
+    ClusterLeaseClaimConfiguration clusterLeaseClaimConfiguration();
 }
