@@ -19,10 +19,7 @@
 package org.apache.cassandra.sidecar.config.yaml;
 
 import java.io.IOException;
-import java.net.URI;
-import java.net.URISyntaxException;
 import java.nio.file.Path;
-import java.nio.file.Paths;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
@@ -38,18 +35,23 @@ import com.fasterxml.jackson.databind.ObjectMapper;
 import com.fasterxml.jackson.databind.module.SimpleModule;
 import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;
 import org.apache.cassandra.sidecar.common.DataObjectBuilder;
+import org.apache.cassandra.sidecar.common.server.dns.DnsResolver;
+import org.apache.cassandra.sidecar.common.server.dns.DnsResolvers;
 import org.apache.cassandra.sidecar.common.server.utils.MillisecondBoundConfiguration;
 import org.apache.cassandra.sidecar.config.AccessControlConfiguration;
 import org.apache.cassandra.sidecar.config.CassandraInputValidationConfiguration;
 import org.apache.cassandra.sidecar.config.DriverConfiguration;
 import org.apache.cassandra.sidecar.config.InstanceConfiguration;
+import org.apache.cassandra.sidecar.config.LiveMigrationConfiguration;
 import org.apache.cassandra.sidecar.config.MetricsConfiguration;
 import org.apache.cassandra.sidecar.config.PeriodicTaskConfiguration;
 import org.apache.cassandra.sidecar.config.RestoreJobConfiguration;
 import org.apache.cassandra.sidecar.config.S3ClientConfiguration;
 import org.apache.cassandra.sidecar.config.SchemaReportingConfiguration;
 import org.apache.cassandra.sidecar.config.ServiceConfiguration;
+import org.apache.cassandra.sidecar.config.SidecarClientConfiguration;
 import org.apache.cassandra.sidecar.config.SidecarConfiguration;
+import org.apache.cassandra.sidecar.config.SidecarPeerHealthConfiguration;
 import org.apache.cassandra.sidecar.config.SslConfiguration;
 import org.apache.cassandra.sidecar.config.VertxConfiguration;
 import org.jetbrains.annotations.NotNull;
@@ -81,6 +83,12 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
     @JsonProperty("access_control")
     protected final AccessControlConfiguration accessControlConfiguration;
 
+    @JsonProperty("sidecar_peer_health")
+    protected final SidecarPeerHealthConfiguration sidecarPeerHealthConfiguration;
+
+    @JsonProperty("sidecar_client")
+    protected final SidecarClientConfiguration sidecarClientConfiguration;
+
     @JsonProperty("healthcheck")
     protected final PeriodicTaskConfiguration healthCheckConfiguration;
 
@@ -104,6 +112,9 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
     @NotNull
     protected final SchemaReportingConfiguration schemaReportingConfiguration;
 
+    @JsonProperty("live_migration")
+    private LiveMigrationConfiguration liveMigrationConfiguration;
+
     public SidecarConfigurationImpl()
     {
         this(builder());
@@ -117,6 +128,8 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
         sslConfiguration = builder.sslConfiguration;
         accessControlConfiguration = builder.accessControlConfiguration;
         healthCheckConfiguration = builder.healthCheckConfiguration;
+        sidecarPeerHealthConfiguration = builder.sidecarPeerHealthConfiguration;
+        sidecarClientConfiguration = builder.sidecarClientConfiguration;
         metricsConfiguration = builder.metricsConfiguration;
         cassandraInputValidationConfiguration = builder.cassandraInputValidationConfiguration;
         driverConfiguration = builder.driverConfiguration;
@@ -124,6 +137,7 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
         s3ClientConfiguration = builder.s3ClientConfiguration;
         vertxConfiguration = builder.vertxConfiguration;
         schemaReportingConfiguration = builder.schemaReportingConfiguration;
+        liveMigrationConfiguration = builder.liveMigrationConfiguration;
     }
 
     /**
@@ -186,6 +200,12 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
         return accessControlConfiguration;
     }
 
+    @Override
+    public SidecarClientConfiguration sidecarClientConfiguration()
+    {
+        return sidecarClientConfiguration;
+    }
+
     /**
      * @return the configuration for the health check service
      */
@@ -196,6 +216,16 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
         return healthCheckConfiguration;
     }
 
+    /**
+     * @return the configuration for the down detector service
+     */
+    @Override
+    @JsonProperty("sidecar_peer_health")
+    public SidecarPeerHealthConfiguration sidecarPeerHealthConfiguration()
+    {
+        return sidecarPeerHealthConfiguration;
+    }
+
     /**
      * @return the configuration needed for metrics capture
      */
@@ -265,18 +295,12 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
         return schemaReportingConfiguration;
     }
 
-    public static SidecarConfigurationImpl readYamlConfiguration(String yamlConfigurationPath) throws IOException
+    public LiveMigrationConfiguration liveMigrationConfiguration()
     {
-        try
-        {
-            return readYamlConfiguration(Paths.get(new URI(yamlConfigurationPath)));
-        }
-        catch (URISyntaxException e)
-        {
-            throw new IOException("Invalid URI: " + yamlConfigurationPath, e);
-        }
+        return liveMigrationConfiguration;
     }
 
+
     public static SidecarConfigurationImpl readYamlConfiguration(Path yamlConfigurationPath) throws IOException
     {
         ObjectMapper mapper = new ObjectMapper(new YAMLFactory())
@@ -289,7 +313,7 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
     }
 
     @VisibleForTesting
-    static SidecarConfigurationImpl fromYamlString(String yaml) throws IOException
+    public static SidecarConfigurationImpl fromYamlString(String yaml) throws IOException
     {
         ObjectMapper mapper = new ObjectMapper(new YAMLFactory())
                               .configure(DeserializationFeature.ACCEPT_SINGLE_VALUE_AS_ARRAY, true)
@@ -348,6 +372,9 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
                      module.addAbstractTypeMapping(configurationInterface, clazz);
                  });
 
+        // Manually add type mapping for DnsResolver
+        module.addAbstractTypeMapping(DnsResolver.class, DnsResolvers.class);
+
         Set<Class> unimplemented = Sets.difference(declared, implemented);
         if (!unimplemented.isEmpty())
         {
@@ -375,6 +402,8 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
         = new PeriodicTaskConfigurationImpl(true,
                                             MillisecondBoundConfiguration.ZERO,
                                             MillisecondBoundConfiguration.parse("30s"));
+        private SidecarPeerHealthConfiguration sidecarPeerHealthConfiguration = new SidecarPeerHealthConfigurationImpl();
+        private SidecarClientConfiguration sidecarClientConfiguration = new SidecarClientConfigurationImpl();
         private MetricsConfiguration metricsConfiguration = new MetricsConfigurationImpl();
         private CassandraInputValidationConfiguration cassandraInputValidationConfiguration = new CassandraInputValidationConfigurationImpl();
         private DriverConfiguration driverConfiguration = new DriverConfigurationImpl();
@@ -382,6 +411,7 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
         private S3ClientConfiguration s3ClientConfiguration = new S3ClientConfigurationImpl();
         private VertxConfiguration vertxConfiguration = new VertxConfigurationImpl();
         private SchemaReportingConfiguration schemaReportingConfiguration = new SchemaReportingConfigurationImpl();
+        private LiveMigrationConfiguration liveMigrationConfiguration = new LiveMigrationConfigurationImpl();
 
         protected Builder()
         {
@@ -459,6 +489,28 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
             return update(b -> b.healthCheckConfiguration = healthCheckConfiguration);
         }
 
+        /**
+         * Sets the {@code healthCheckConfiguration} and returns a reference to this Builder enabling method chaining.
+         *
+         * @param sidecarPeerHealthConfiguration the {@code healthCheckConfiguration} to set
+         * @return a reference to this Builder
+         */
+        public Builder sidecarPeerHealthConfiguration(SidecarPeerHealthConfiguration sidecarPeerHealthConfiguration)
+        {
+            return update(b -> b.sidecarPeerHealthConfiguration = sidecarPeerHealthConfiguration);
+        }
+
+        /**
+         * Sets the {@code sidecarClientConfiguration} and returns a reference to this Builder enabling method chaining.
+         *
+         * @param sidecarClientConfiguration the {@code sidecarClientConfiguration} to set
+         * @return a reference to this Builder
+         */
+        public Builder sidecarClientConfiguration(SidecarClientConfiguration sidecarClientConfiguration)
+        {
+            return update(b -> b.sidecarClientConfiguration = sidecarClientConfiguration);
+        }
+
         /**
          * Sets the {@code metricsConfiguration} and returns a reference to this Builder enabling method chaining.
          *
@@ -541,6 +593,18 @@ public class SidecarConfigurationImpl implements SidecarConfiguration
             return update(builder -> builder.schemaReportingConfiguration = configuration);
         }
 
+        /**
+         * Sets the {@code liveMigrationConfiguration} and returns a reference to this Builder enabling method
+         * chaining
+         *
+         * @param liveMigrationConfiguration the {@code liveMigrationConfiguration} to set
+         * @return a reference to this Builder
+         */
+        public Builder liveMigrationConfiguration(LiveMigrationConfiguration liveMigrationConfiguration)
+        {
+            return update(b -> b.liveMigrationConfiguration = liveMigrationConfiguration);
+        }
+
         /**
          * Returns a {@code SidecarConfigurationImpl} built from the parameters previously set.
          *
