@@ -40,15 +40,15 @@ import com.google.inject.Inject;
 import com.google.inject.Provider;
 import com.google.inject.Singleton;
 import io.vertx.core.Promise;
+import io.vertx.core.Vertx;
 import org.apache.cassandra.sidecar.cluster.instance.InstanceMetadata;
 import org.apache.cassandra.sidecar.cluster.locator.LocalTokenRangesProvider;
 import org.apache.cassandra.sidecar.common.data.RestoreJobStatus;
+import org.apache.cassandra.sidecar.common.response.NodeSettings;
 import org.apache.cassandra.sidecar.common.response.TokenRangeReplicasResponse;
 import org.apache.cassandra.sidecar.common.server.cluster.locator.TokenRange;
 import org.apache.cassandra.sidecar.common.server.utils.DurationSpec;
 import org.apache.cassandra.sidecar.common.utils.Preconditions;
-import org.apache.cassandra.sidecar.concurrent.ExecutorPools;
-import org.apache.cassandra.sidecar.concurrent.TaskExecutorPool;
 import org.apache.cassandra.sidecar.config.RestoreJobConfiguration;
 import org.apache.cassandra.sidecar.config.SidecarConfiguration;
 import org.apache.cassandra.sidecar.db.RestoreJob;
@@ -58,9 +58,10 @@ import org.apache.cassandra.sidecar.db.RestoreRangeDatabaseAccessor;
 import org.apache.cassandra.sidecar.db.RestoreSlice;
 import org.apache.cassandra.sidecar.db.RestoreSliceDatabaseAccessor;
 import org.apache.cassandra.sidecar.db.schema.SidecarSchema;
+import org.apache.cassandra.sidecar.exceptions.CassandraUnavailableException;
 import org.apache.cassandra.sidecar.exceptions.RestoreJobFatalException;
-import org.apache.cassandra.sidecar.metrics.RestoreMetrics;
 import org.apache.cassandra.sidecar.metrics.SidecarMetrics;
+import org.apache.cassandra.sidecar.metrics.server.RestoreMetrics;
 import org.apache.cassandra.sidecar.tasks.PeriodicTask;
 import org.apache.cassandra.sidecar.tasks.PeriodicTaskExecutor;
 import org.apache.cassandra.sidecar.tasks.ScheduleDecision;
@@ -86,7 +87,7 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
     private final JobIdsByDay jobIdsByDay;
     private final RingTopologyRefresher ringTopologyRefresher;
     private final AtomicBoolean isExecuting = new AtomicBoolean(false);
-    private final TaskExecutorPool executorPool;
+    private String localDatacenter = null;
     private int inflightJobsCount = 0;
     private int jobDiscoveryRecencyDays;
     private PeriodicTaskExecutor periodicTaskExecutor;
@@ -100,7 +101,6 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
                                 Provider<RestoreJobManagerGroup> restoreJobManagerGroupProvider,
                                 InstanceMetadataFetcher instanceMetadataFetcher,
                                 RingTopologyRefresher ringTopologyRefresher,
-                                ExecutorPools executorPools,
                                 SidecarMetrics metrics)
     {
         this(config.restoreJobConfiguration(),
@@ -111,7 +111,7 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
              restoreJobManagerGroupProvider,
              instanceMetadataFetcher,
              ringTopologyRefresher,
-             executorPools,
+             null,
              metrics);
     }
 
@@ -124,7 +124,9 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
                          Provider<RestoreJobManagerGroup> restoreJobManagerGroupProvider,
                          InstanceMetadataFetcher instanceMetadataFetcher,
                          RingTopologyRefresher ringTopologyRefresher,
-                         ExecutorPools executorPools,
+                         // Only set when testing; Do not add PeriodicTaskExecutor to constructor, as it creates circular dependency
+                         // this.periodicTaskExecutor is set when deploying the task to PeriodicTaskExecutor
+                         PeriodicTaskExecutor executor,
                          SidecarMetrics metrics)
     {
         this.restoreJobConfig = restoreJobConfig;
@@ -138,8 +140,15 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
         this.ringTopologyRefresher = ringTopologyRefresher;
         this.localTokenRangesProvider = ringTopologyRefresher;
         this.metrics = metrics.server().restore();
+        this.periodicTaskExecutor = executor;
         this.jobIdsByDay = new JobIdsByDay();
-        this.executorPool = executorPools.internal();
+    }
+
+    @Override
+    public void deploy(Vertx vertx, PeriodicTaskExecutor executor)
+    {
+        this.periodicTaskExecutor = executor;
+        PeriodicTask.super.deploy(vertx, executor);
     }
 
     @Override
@@ -161,6 +170,7 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
     @Override
     public void execute(Promise<Void> promise)
     {
+        initLocalDatacenterMaybe();
         tryExecuteDiscovery();
         promise.tryComplete();
     }
@@ -242,12 +252,6 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
         metrics.activeJobs.metric.setValue(inflightJobsCount);
     }
 
-    @Override
-    public void registerPeriodicTaskExecutor(PeriodicTaskExecutor executor)
-    {
-        this.periodicTaskExecutor = executor;
-    }
-
     @Override
     public void onRingTopologyChanged(String keyspace, TokenRangeReplicasResponse oldTopology, TokenRangeReplicasResponse newTopology)
     {
@@ -375,6 +379,15 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
             return;
         }
 
+        // stop proceeding further if the local datacenter is excluded from the restore job
+        if (isLocalDatacenterExcluded(job))
+        {
+            LOGGER.info("Restore job is configured to skip running on the local datacenter. " +
+                        "jobId={} localDatacenter={} targetDatacenter={}",
+                        job.jobId, localDatacenter, job.localDatacenter);
+            return;
+        }
+
         // Only force refresh topology for the first time in each stage
         // RestoreJobDiscoverer is registered as a RingTopologyListener to receive future topology changed notifications, if any
         ringTopologyRefresher.register(job, this);
@@ -386,6 +399,41 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
         }
     }
 
+    private void initLocalDatacenterMaybe()
+    {
+        if (localDatacenter != null)
+        {
+            return;
+        }
+
+        try
+        {
+            NodeSettings nodeSettings = instanceMetadataFetcher.callOnFirstAvailableInstance(i -> i.delegate().nodeSettings());
+            localDatacenter = nodeSettings.datacenter();
+        }
+        catch (CassandraUnavailableException cue)
+        {
+            LOGGER.debug("localDatacenter is not initialized", cue);
+        }
+    }
+
+    private boolean isLocalDatacenterExcluded(RestoreJob job)
+    {
+        if (!job.shouldRestoreToLocalDatacenterOnly)
+        {
+            return false;
+        }
+
+        if (localDatacenter == null)
+        {
+            LOGGER.debug("The restore job should restore only to the local datacenter, but the local datacenter is undetermined yet; skip this run");
+            return true;
+        }
+
+        // when job should restore to local datacenter only, but the target datacenter is not the local one
+        return !Objects.equals(localDatacenter, job.localDatacenter);
+    }
+
     private boolean shouldFindSlicesAndSubmit(RestoreJob job)
     {
         return (job.status == RestoreJobStatus.STAGE_READY || job.status == RestoreJobStatus.IMPORT_READY)
@@ -475,10 +523,8 @@ public class RestoreJobDiscoverer implements PeriodicTask, RingTopologyChangeLis
         }
         catch (RestoreJobFatalException e)
         {
-            LOGGER.error("Restore range failed. startToken={} endToken={} instance={}",
-                         range.startToken(), range.endToken(), range.owner().host(), e);
-            range.fail(e);
-            restoreRangeDatabaseAccessor.updateStatus(range);
+            LOGGER.error("The restore job has already failed. jobId={} startToken={} endToken={} instance={}",
+                         job.jobId, range.startToken(), range.endToken(), range.owner().host(), e);
             return RestoreJobProgressTracker.Status.FAILED;
         }
     }
