@@ -27,7 +27,6 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
-
 import com.google.common.annotations.VisibleForTesting;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -48,10 +47,10 @@ import org.apache.cassandra.sidecar.db.RestoreRangeDatabaseAccessor;
 import org.apache.cassandra.sidecar.exceptions.RestoreJobException;
 import org.apache.cassandra.sidecar.exceptions.RestoreJobExceptions;
 import org.apache.cassandra.sidecar.exceptions.RestoreJobFatalException;
-import org.apache.cassandra.sidecar.metrics.RestoreMetrics;
 import org.apache.cassandra.sidecar.metrics.SidecarMetrics;
 import org.apache.cassandra.sidecar.metrics.StopWatch;
 import org.apache.cassandra.sidecar.metrics.instance.InstanceMetrics;
+import org.apache.cassandra.sidecar.metrics.server.RestoreMetrics;
 import org.apache.cassandra.sidecar.utils.SSTableImporter;
 import software.amazon.awssdk.core.exception.ApiCallTimeoutException;
 import software.amazon.awssdk.services.s3.model.NoSuchKeyException;
@@ -172,9 +171,25 @@ public class RestoreRangeTask implements RestoreRangeHandler
                 }
                 else if (job.status == RestoreJobStatus.IMPORT_READY)
                 {
-                    return unzipAndImport(range.stagedObjectPath().toFile(),
-                                          // persist status
-                                          () -> rangeDatabaseAccessor.updateStatus(range));
+                    if (range.hasStaged())
+                    {
+                        return unzipAndImport(range.stagedObjectPath().toFile(),
+                                              // persist status
+                                              () -> rangeDatabaseAccessor.updateStatus(range));
+                    }
+                    else
+                    {
+                        LOGGER.info("The restore job is in {} status, but the range has not been staged yet. " +
+                                    "Perhaps the staged slices in the cluster already satisfy the consistency level of the job. " +
+                                    "This sidecar instance would attempt to stage the slice and import for improved consistency. " +
+                                    "jobId={} consistencyLevel={} startToken={} endToken={} sliceKey={}",
+                                    job.status, job.jobId, job.consistencyLevel, range.startToken(), range.endToken(), range.sliceKey());
+                        return downloadSliceAndImport()
+                               .compose(ignored -> { // persist status
+                                   rangeDatabaseAccessor.updateStatus(range);
+                                   return Future.succeededFuture();
+                               });
+                    }
                 }
                 else
                 {
