@@ -56,7 +56,7 @@ import static org.assertj.core.api.Assumptions.assumeThat;
 @ExtendWith(VertxExtension.class)
 class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
 {
-    private static final int MIN_VERSION_WITH_MTLS = 5;
+    protected static final int MIN_VERSION_WITH_MTLS = 5;
 
     private Path nonAdminClientKeystorePath;
     private CountDownLatch testCompleteLatch;
@@ -69,13 +69,14 @@ class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
         // wait for cache refreshes
         Thread.sleep(3000);
 
-        testCompleteLatch = new CountDownLatch(32);
+        testCompleteLatch = new CountDownLatch(36);
 
         // permissions for test cases below are granted during prepareForTest to save cache refresh time. Please
         // refer to grantRequiredPermissions to check permissions granted for a test to understand verifications done in
         // test
         testForAdmin(context);
         testForSuperUser(context);
+        testForTransitiveSuperUser(context);
         testForNonAdmin(context);
         testGrantingForTable(context);
         testGrantingForKeyspace(context);
@@ -112,6 +113,15 @@ class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
         verifyAccess(context, testCompleteLatch, HttpMethod.GET, keyspaceSchemaRoute, clientKeystorePath, false);
     }
 
+    void testForTransitiveSuperUser(VertxTestContext context) throws Exception
+    {
+        String keyspaceSchemaRoute = String.format("/api/v1/keyspaces/%s/schema", "test_keyspace");
+        // uses client keystore with superuser identity
+        Path clientKeystorePath = clientKeystorePath("spiffe://cassandra/sidecar/non_super_user_with_transitive_super_user");
+
+        verifyAccess(context, testCompleteLatch, HttpMethod.GET, keyspaceSchemaRoute, clientKeystorePath, false);
+    }
+
     void testForNonAdmin(VertxTestContext context)
     {
         String keyspaceSchemaRoute = String.format("/api/v1/keyspaces/%s/schema", "non_admin_test_keyspace");
@@ -246,7 +256,7 @@ class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
         })
         .onSuccess(acceptedStreamResp ->  {
             // stream request goes through with both SNAPSHOT:STREAM and SELECT permissions
-            assertThat(acceptedStreamResp.statusCode()).isEqualTo(HttpResponseStatus.OK.code());
+            context.verify(() -> assertThat(acceptedStreamResp.statusCode()).isEqualTo(HttpResponseStatus.OK.code()));
             testCompleteLatch.countDown();
         })
         .onFailure(context::failNow);
@@ -270,6 +280,15 @@ class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
         String topologyRoute = String.format("/api/v1/keyspaces/%s/token-range-replicas", "grant_bulk_read_test_keyspace");
         // TOPOLOGY:READ permission not granted with ANALYTICS:READ_DIRECT
         verifyAccess(context, testCompleteLatch, HttpMethod.GET, topologyRoute, clientKeystorePath, true);
+
+        String tableStatsRoute = String.format("/api/v1/cassandra/keyspaces/%s/tables/%s/stats",
+                                               "grant_bulk_read_test_keyspace", "test_table");
+        // STATS permission granted with ANALYTICS:READ_DIRECT
+        verifyAccess(context, testCompleteLatch, HttpMethod.GET, tableStatsRoute, clientKeystorePath, false);
+
+        String clusterStatsRoute = "/api/v1/cassandra/stats/streams";
+        // STATS permission at cluster scope not granted with ANALYTICS:READ_DIRECT
+        verifyAccess(context, testCompleteLatch, HttpMethod.GET, clusterStatsRoute, clientKeystorePath, true);
     }
 
     void testGrantingBulkReadFeaturePermissionAcrossTables(VertxTestContext context) throws Exception
@@ -293,6 +312,11 @@ class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
 
         // SNAPSHOT:CREATE for different keyspace not granted with keyspace scoped ANALYTICS:READ_DIRECT permission
         verifyAccess(context, testCompleteLatch, HttpMethod.PUT, createSnapshotRouteKeyspace2, clientKeystorePath, true);
+
+        String tableStatsRoute = String.format("/api/v1/cassandra/keyspaces/%s/tables/%s/stats",
+                                               "grant_bulk_read_across_tables_test_keyspace", "test_table");
+        // STATS permission granted with ANALYTICS:READ_DIRECT
+        verifyAccess(context, testCompleteLatch, HttpMethod.GET, tableStatsRoute, clientKeystorePath, false);
     }
 
     void testGrantingBulkReadFeaturePermissionAcrossData(VertxTestContext context) throws Exception
@@ -371,8 +395,8 @@ class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
         .onSuccess(listResp -> {
             // CDC permission granted
             // CDC is not turned on for cluster, hence 500 or 503 expected
-            assertThat(listResp.statusCode()).isIn(HttpResponseStatus.SERVICE_UNAVAILABLE.code(),
-                                                   HttpResponseStatus.INTERNAL_SERVER_ERROR.code());
+            context.verify(() -> assertThat(listResp.statusCode()).isIn(HttpResponseStatus.SERVICE_UNAVAILABLE.code(),
+                                                                        HttpResponseStatus.INTERNAL_SERVER_ERROR.code()));
             testCompleteLatch.countDown();
         });
     }
@@ -433,6 +457,10 @@ class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
         createRole("super_user_test_role", true);
         insertIdentityRole(cassandraContext, "spiffe://cassandra/sidecar/super_user_test_user", "super_user_test_role");
 
+        createRole("non_super_user_with_transitive_super_user_role", false);
+        grantRole("non_super_user_with_transitive_super_user_role", "super_user_test_role");
+        insertIdentityRole(cassandraContext, "spiffe://cassandra/sidecar/non_super_user_with_transitive_super_user", "non_super_user_with_transitive_super_user_role");
+
         createRole("non_admin_test_role", false);
         insertIdentityRole(cassandraContext, "spiffe://cassandra/sidecar/non_admin_test_user", "non_admin_test_role");
 
@@ -516,12 +544,6 @@ class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
         nonAdminClientKeystorePath = clientKeystorePath("spiffe://cassandra/sidecar/non_admin_test_user");
     }
 
-    private void createRole(String role, boolean superUser)
-    {
-        Session session = maybeGetSession();
-        session.execute("CREATE ROLE " + role + " WITH PASSWORD = 'password' AND SUPERUSER = " + superUser + " AND LOGIN = true;");
-    }
-
     private void createKeyspace(String keyspace)
     {
         Session session = maybeGetSession();
@@ -587,7 +609,7 @@ class RoleBasedAuthorizationIntegrationTest extends IntegrationTestBase
               });
     }
 
-    private  Future<HttpResponse<Buffer>> createReq(WebClient client, HttpMethod method, String route)
+    private Future<HttpResponse<Buffer>> createReq(WebClient client, HttpMethod method, String route)
     {
         return client.request(method, server.actualPort(), "127.0.0.1", route).send();
     }
