@@ -64,11 +64,12 @@ import org.apache.cassandra.sidecar.cluster.InstancesMetadata;
 import org.apache.cassandra.sidecar.cluster.instance.InstanceMetadata;
 import org.apache.cassandra.sidecar.common.server.data.Name;
 import org.apache.cassandra.sidecar.common.server.data.QualifiedTableName;
-import org.apache.cassandra.sidecar.common.server.dns.DnsResolver;
+import org.apache.cassandra.sidecar.common.server.dns.DnsResolvers;
 import org.apache.cassandra.sidecar.config.SslConfiguration;
 import org.apache.cassandra.sidecar.config.yaml.KeyStoreConfigurationImpl;
 import org.apache.cassandra.sidecar.config.yaml.SslConfigurationImpl;
-import org.apache.cassandra.sidecar.server.MainModule;
+import org.apache.cassandra.sidecar.db.schema.TableSchemaFetcher;
+import org.apache.cassandra.sidecar.modules.SidecarModules;
 import org.apache.cassandra.sidecar.server.Server;
 import org.apache.cassandra.sidecar.server.SidecarServerEvents;
 import org.apache.cassandra.testing.AbstractCassandraTestContext;
@@ -135,8 +136,7 @@ public abstract class IntegrationTestBase
         System.setProperty("suitename", testInfo.getDisplayName() + ": " + cassandraTestContext.version);
         int clusterSize = cassandraTestContext.clusterSize();
         // list of modules that override the priors; hence order matters
-        List<Module> modules = new ArrayList<>();
-        modules.add(new MainModule());
+        List<Module> modules = new ArrayList<>(SidecarModules.all());
         modules.add(integrationTestModule);
         if (testSpecificModule != null)
         {
@@ -158,9 +158,10 @@ public abstract class IntegrationTestBase
         {
             sslConfig = sslConfigWithTruststore();
         }
-        sidecarTestContext = CassandraSidecarTestContext.from(vertx, cassandraTestContext, DnsResolver.DEFAULT,
-                                                              getInstancesToManage(clusterSize), sslConfig);
-        integrationTestModule.setCassandraTestContext(sidecarTestContext);
+        TableSchemaFetcher tableSchemaFetcher = injector.getInstance(TableSchemaFetcher.class);
+        sidecarTestContext = CassandraSidecarTestContext.from(vertx, cassandraTestContext, DnsResolvers.DEFAULT,
+                                                              getInstancesToManage(clusterSize), sslConfig, tableSchemaFetcher);
+        integrationTestModule.setCassandraSidecarTestContext(sidecarTestContext);
 
         server = injector.getInstance(Server.class);
         VertxTestContext context = new VertxTestContext();
@@ -389,6 +390,23 @@ public abstract class IntegrationTestBase
         return udt;
     }
 
+    protected void createRole(String role, boolean superUser)
+    {
+        createRole(role, "password", superUser);
+    }
+
+    protected void createRole(String role, String password, boolean superUser)
+    {
+        Session session = maybeGetSession();
+        session.execute("CREATE ROLE \"" + role + "\" WITH PASSWORD ='" + password + "' AND SUPERUSER = " + superUser + " AND LOGIN = true;");
+    }
+
+    protected void grantRole(String role, String roleToAssign)
+    {
+        Session session = maybeGetSession();
+        session.execute("GRANT " + roleToAssign + " TO " + role + ";");
+    }
+
     // similar to awaitLatchOrTimeout, it throws either test exceptions (due to startAsync failures) or timeout exception
     public void awaitLatchOrThrow(CountDownLatch latch, long duration, TimeUnit timeUnit, String latchName)
     {
@@ -552,6 +570,6 @@ public abstract class IntegrationTestBase
         options.setSsl(true);
         options.setKeyStoreOptions(new JksOptions().setPath(clientKeystorePath.toAbsolutePath().toString()).setPassword(clientKeystorePassword));
         options.setTrustStoreOptions(new JksOptions().setPath(truststorePath.toAbsolutePath().toString()).setPassword(truststorePassword));
-        return WebClient.create(vertx, options);
+        return WebClient.create(vertx, options); // TODO: webclient is not closed
     }
 }
