@@ -30,6 +30,8 @@ import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.io.TempDir;
 
 import com.fasterxml.jackson.databind.JsonMappingException;
+import org.apache.cassandra.sidecar.common.server.dns.DnsResolver;
+import org.apache.cassandra.sidecar.common.server.utils.MillisecondBoundConfiguration;
 import org.apache.cassandra.sidecar.config.yaml.MetricsFilteringConfigurationImpl;
 import org.apache.cassandra.sidecar.config.yaml.SidecarConfigurationImpl;
 import org.assertj.core.api.Condition;
@@ -227,7 +229,7 @@ class SidecarConfigurationTest
         assertThat(configuration.replicationFactor()).isEqualTo(3);
         assertThat(configuration.createReplicationStrategyString())
         .isEqualTo("{'class':'SimpleStrategy', 'replication_factor':'3'}");
-        assertThat(configuration.leaseSchemaTTL().toSeconds()).isEqualTo(120);
+        assertThat(configuration.leaseSchemaTTL().toSeconds()).isEqualTo(300L);
     }
 
     @Test
@@ -293,6 +295,66 @@ class SidecarConfigurationTest
         assertThat(pattern.matcher("throttled_429")).matches();
     }
 
+    @Test
+    void testSidecarPeerHealthConfiguration() throws IOException
+    {
+        String yaml = "sidecar_peer_health:\n" +
+                      "  enabled: true\n" +
+                      "  execute_interval: 63s\n" +
+                      "  max_retries: 15\n" +
+                      "  retry_delay: 5s";
+        SidecarConfigurationImpl sidecarConfiguration = SidecarConfigurationImpl.fromYamlString(yaml);
+        assertThat(sidecarConfiguration).isNotNull();
+        SidecarPeerHealthConfiguration config = sidecarConfiguration.sidecarPeerHealthConfiguration();
+        assertThat(config).isNotNull();
+        assertThat(config.enabled()).isTrue();
+        assertThat(config.executeInterval().toMillis()).isEqualTo(63_000);
+        assertThat(config.maxRetries()).isEqualTo(15);
+        assertThat(config.retryDelay().toMillis()).isEqualTo(5_000);
+    }
+
+    @Test
+    void testSidecarClientConfiguration() throws IOException
+    {
+        String yaml = "sidecar_client:\n" +
+                      "  request_timeout: 1s\n" +
+                      "  request_idle_timeout: 1s\n" +
+                      "  connection_pool_max_size: 10\n" +
+                      "  connection_pool_clearing_period: 10s\n" +
+                      "  connection_pool_event_loop_size: 10\n" +
+                      "  connection_pool_max_wait_queue_size: 10\n" +
+                      "  max_retries: 3\n" +
+                      "  retry_delay: 1s\n" +
+                      "  max_retry_delay: 2s\n" +
+                      "  ssl:\n" +
+                      "    enabled: false\n" +
+                      "    keystore:\n" +
+                      "      type: PKCS12\n" +
+                      "      path: path/to/keystore.p12\n" +
+                      "      password: password\n" +
+                      "    truststore:\n" +
+                      "      type: PKCS12\n" +
+                      "      path: path/to/keystore.p12\n" +
+                      "      password: password";
+        SidecarConfigurationImpl sidecarConfiguration = SidecarConfigurationImpl.fromYamlString(yaml);
+        assertThat(sidecarConfiguration).isNotNull();
+        SidecarClientConfiguration config = sidecarConfiguration.sidecarClientConfiguration();
+        assertThat(config).isNotNull();
+        assertThat(config.sslConfiguration()).isNotNull();
+        assertThat(config.sslConfiguration().enabled()).isFalse();
+        assertThat(config.sslConfiguration().isKeystoreConfigured()).isTrue();
+        assertThat(config.sslConfiguration().isTrustStoreConfigured()).isTrue();
+        assertThat(config.requestTimeout().toMillis()).isEqualTo(1_000);
+        assertThat(config.requestIdleTimeout().toMillis()).isEqualTo(1_000);
+        assertThat(config.connectionPoolMaxSize()).isEqualTo(10);
+        assertThat(config.connectionPoolCleanerPeriod().toMillis()).isEqualTo(10_000);
+        assertThat(config.connectionPoolEventLoopSize()).isEqualTo(10);
+        assertThat(config.connectionPoolMaxWaitQueueSize()).isEqualTo(10);
+        assertThat(config.maxRetries()).isEqualTo(3);
+        assertThat(config.retryDelay().toMillis()).isEqualTo(1_000);
+        assertThat(config.maxRetryDelay().toMillis()).isEqualTo(2_000);
+    }
+
     @Test
     void testVertxFilesystemOptionsConfiguration() throws IOException
     {
@@ -318,6 +380,8 @@ class SidecarConfigurationTest
         CdcConfiguration cdcConfig = sidecarConfiguration.serviceConfiguration().cdcConfiguration();
         assertThat(cdcConfig.segmentHardLinkCacheExpiry().quantity()).isEqualTo(1);
         assertThat(cdcConfig.segmentHardLinkCacheExpiry().unit()).isEqualTo(TimeUnit.MINUTES);
+        assertThat(cdcConfig.isEnabled()).isEqualTo(true);
+        assertThat(cdcConfig.cdcConfigRefreshTime()).isEqualTo(MillisecondBoundConfiguration.parse("10s"));
     }
 
     @Test
@@ -331,11 +395,16 @@ class SidecarConfigurationTest
         assertThat(accessControlConfiguration.enabled()).isTrue();
 
         List<ParameterizedClassConfiguration> authenticators = accessControlConfiguration.authenticatorsConfiguration();
-        assertThat(authenticators).isNotNull().hasSize(1);
+        assertThat(authenticators).isNotNull().hasSize(2);
         assertThat(authenticators.get(0).className()).isEqualTo("org.apache.cassandra.sidecar.acl.authentication.MutualTLSAuthenticationHandlerFactory");
         assertThat(authenticators.get(0).namedParameters())
         .contains(entry("certificate_validator", "io.vertx.ext.auth.mtls.impl.AllowAllCertificateValidator"),
                   entry("certificate_identity_extractor", "org.apache.cassandra.sidecar.acl.authentication.CassandraIdentityExtractor"));
+        assertThat(authenticators.get(1).className()).isEqualTo("org.apache.cassandra.sidecar.acl.authentication.JwtAuthenticationHandlerFactory");
+        assertThat(authenticators.get(1).namedParameters())
+        .contains(entry("site", "https://authorization.com"),
+                  entry("client_id", "recognized_client_id"),
+                  entry("config_discover_interval", "1m"));
 
         ParameterizedClassConfiguration authorizer = accessControlConfiguration.authorizerConfiguration();
         assertThat(authorizer.className()).isEqualTo("org.apache.cassandra.sidecar.acl.authorization.RoleBasedAuthorizationProvider");
@@ -359,17 +428,99 @@ class SidecarConfigurationTest
     @Test
     void testCoordinationConfiguration() throws Exception
     {
-        Path yamlPath = yaml("config/sidecar_coordination.yaml");
-        SidecarConfiguration config = SidecarConfigurationImpl.readYamlConfiguration(yamlPath);
+        String yaml = "sidecar:\n" +
+                      "  coordination:\n" +
+                      "    cluster_lease_claim:\n" +
+                      "      electorate_membership_strategy: SidecarInternalTokenZeroElectorateMembership\n" +
+                      "      enabled: false\n" +
+                      "      initial_delay: 5s\n" +
+                      "      execute_interval: 31s\n" +
+                      "      initial_delay_random_delta: 10s";
+        SidecarConfiguration config = SidecarConfigurationImpl.fromYamlString(yaml);
         ServiceConfiguration serviceConfiguration = config.serviceConfiguration();
         assertThat(serviceConfiguration).isNotNull();
 
         CoordinationConfiguration coordinationConfiguration = serviceConfiguration.coordinationConfiguration();
         assertThat(coordinationConfiguration).isNotNull();
-        PeriodicTaskConfiguration periodicTaskConfig = coordinationConfiguration.clusterLeaseClaimConfiguration();
-        assertThat(periodicTaskConfig.enabled()).isFalse();
-        assertThat(periodicTaskConfig.initialDelay().toMillis()).isEqualTo(5_000L);
-        assertThat(periodicTaskConfig.executeInterval().toMillis()).isEqualTo(31_000L);
+        ClusterLeaseClaimConfiguration clusterLeaseConfig = coordinationConfiguration.clusterLeaseClaimConfiguration();
+        assertThat(clusterLeaseConfig.enabled()).isFalse();
+        assertThat(clusterLeaseConfig.initialDelay().toMillis()).isEqualTo(5_000L);
+        assertThat(clusterLeaseConfig.executeInterval().toMillis()).isEqualTo(31_000L);
+        assertThat(clusterLeaseConfig.initialDelayRandomDelta().toMillis()).isEqualTo(10_000L);
+        assertThat(clusterLeaseConfig.randomDeltaDelayMillis()).isBetween(0L, 10_000L);
+        assertThat(clusterLeaseConfig.electorateMembershipStrategy()).isEqualTo("SidecarInternalTokenZeroElectorateMembership");
+    }
+
+    @Test
+    void testCoordinationDefaultElectorateMembershipStrategy() throws Exception
+    {
+        String yaml = "sidecar:\n" +
+                      "  coordination:\n" +
+                      "    cluster_lease_claim:\n" +
+                      "      enabled: false\n" +
+                      "      initial_delay: 5s\n" +
+                      "      execute_interval: 31s";
+        SidecarConfiguration config = SidecarConfigurationImpl.fromYamlString(yaml);
+        ServiceConfiguration serviceConfiguration = config.serviceConfiguration();
+        assertThat(serviceConfiguration).isNotNull();
+
+        CoordinationConfiguration coordinationConfiguration = serviceConfiguration.coordinationConfiguration();
+        assertThat(coordinationConfiguration).isNotNull();
+        ClusterLeaseClaimConfiguration clusterLeaseConfig = coordinationConfiguration.clusterLeaseClaimConfiguration();
+        assertThat(clusterLeaseConfig.enabled()).isFalse();
+        assertThat(clusterLeaseConfig.initialDelay().toMillis()).isEqualTo(5_000L);
+        assertThat(clusterLeaseConfig.executeInterval().toMillis()).isEqualTo(31_000L);
+        assertThat(clusterLeaseConfig.electorateMembershipStrategy()).isEqualTo("MostReplicatedKeyspaceTokenZeroElectorateMembership");
+    }
+
+    @Test
+    void testDnsResolverDefault() throws Exception
+    {
+        Path yamlPath = yaml("config/sidecar_single_instance.yaml");
+        SidecarConfiguration config = SidecarConfigurationImpl.readYamlConfiguration(yamlPath);
+        ServiceConfiguration serviceConfiguration = config.serviceConfiguration();
+        assertThat(serviceConfiguration).isNotNull();
+
+        DnsResolver dnsResolver = serviceConfiguration.dnsResolver();
+        assertThat(dnsResolver).isNotNull();
+        assertThat(dnsResolver.resolve("localhost")).isEqualTo("127.0.0.1");
+        assertThat(dnsResolver.reverseResolve("127.0.0.1")).isEqualTo("localhost");
+    }
+
+    @Test
+    void testDnsResolverResolveToIp() throws Exception
+    {
+        String yaml = "sidecar:\n" +
+                "  dns_resolver: resolve_to_ip";
+        SidecarConfiguration config = SidecarConfigurationImpl.fromYamlString(yaml);
+        ServiceConfiguration serviceConfiguration = config.serviceConfiguration();
+        assertThat(serviceConfiguration).isNotNull();
+
+        DnsResolver dnsResolver = serviceConfiguration.dnsResolver();
+        assertThat(dnsResolver).isNotNull();
+        assertThat(dnsResolver.resolve("localhost")).isEqualTo("127.0.0.1");
+        assertThat(dnsResolver.reverseResolve("127.0.0.1")).isEqualTo("127.0.0.1");
+    }
+
+    @Test
+    void testSidecarLiveMigrationConfiguration() throws IOException
+    {
+        Path yamlPath = yaml("config/sidecar_live_migration.yaml");
+        SidecarConfiguration config = SidecarConfigurationImpl.readYamlConfiguration(yamlPath);
+        LiveMigrationConfiguration liveMigrationConfiguration = config.liveMigrationConfiguration();
+        assertThat(liveMigrationConfiguration).isNotNull();
+
+        assertThat(liveMigrationConfiguration.filesToExclude()).isNotNull()
+                                                               .hasSize(1)
+                                                               .contains("${DATA_FILE_DIR}/test.log");
+
+        assertThat(liveMigrationConfiguration.directoriesToExclude()).isNotNull()
+                                                                     .hasSize(1)
+                                                                     .contains("${DATA_FILE_DIR}/*/*/snapshots");
+
+        assertThat(liveMigrationConfiguration.migrationMap()).isNotNull()
+                                                             .hasSize(1)
+                                                             .containsEntry("localhost1", "localhost4");
     }
 
     void validateSingleInstanceSidecarConfiguration(SidecarConfiguration config)
@@ -402,6 +553,8 @@ class SidecarConfigurationTest
         // metrics configuration
         validateMetricsConfiguration(config.metricsConfiguration());
 
+        validateSidecarPeerHealthConfigurationDefaults(config.sidecarPeerHealthConfiguration());
+
         // cassandra input validation configuration
         validateCassandraInputValidationConfigurationFromYaml(config.cassandraInputValidationConfiguration());
 
@@ -485,6 +638,15 @@ class SidecarConfigurationTest
         validateCassandraInputValidationConfigurationFromYaml(config.cassandraInputValidationConfiguration());
     }
 
+    private void validateSidecarPeerHealthConfigurationDefaults(SidecarPeerHealthConfiguration config)
+    {
+        assertThat(config).isNotNull();
+        assertThat(config.enabled()).isFalse();
+        assertThat(config.executeInterval().toMillis()).isEqualTo(30_000);
+        assertThat(config.maxRetries()).isEqualTo(5);
+        assertThat(config.retryDelay().toMillis()).isEqualTo(10_000);
+    }
+
     void validateServiceConfigurationFromYaml(ServiceConfiguration serviceConfiguration)
     {
         assertThat(serviceConfiguration).isNotNull();
@@ -543,10 +705,13 @@ class SidecarConfigurationTest
         // Validate default configuration
         CoordinationConfiguration coordinationConfiguration = serviceConfiguration.coordinationConfiguration();
         assertThat(coordinationConfiguration).isNotNull();
-        PeriodicTaskConfiguration periodicTaskConfig = coordinationConfiguration.clusterLeaseClaimConfiguration();
-        assertThat(periodicTaskConfig.enabled()).isTrue();
-        assertThat(periodicTaskConfig.executeInterval().toMillis()).isEqualTo(60_000L);
-        assertThat(periodicTaskConfig.initialDelay().toMillis()).isEqualTo(1_000L);
+        ClusterLeaseClaimConfiguration clusterLeaseConfig = coordinationConfiguration.clusterLeaseClaimConfiguration();
+        assertThat(clusterLeaseConfig.enabled()).isTrue();
+        assertThat(clusterLeaseConfig.executeInterval().toMillis()).isEqualTo(100_000L);
+        assertThat(clusterLeaseConfig.initialDelay().toMillis()).isEqualTo(1_000L);
+        assertThat(clusterLeaseConfig.initialDelayRandomDelta().toMillis()).isEqualTo(30_000L);
+        assertThat(clusterLeaseConfig.randomDeltaDelayMillis()).isBetween(0L, 30_000L);
+        assertThat(clusterLeaseConfig.electorateMembershipStrategy()).isEqualTo("MostReplicatedKeyspaceTokenZeroElectorateMembership");
     }
 
     private void validateHealthCheckConfigurationFromYaml(PeriodicTaskConfiguration config)
@@ -581,7 +746,7 @@ class SidecarConfigurationTest
     void validateVertxFilesystemOptionsClasspathResolvingDisabled(FileSystemOptionsConfiguration config)
     {
         assertThat(config).isNotNull();
-        assertThat(config.classpathResolvingEnabled()).isFalse();
+        assertThat(config.classpathResolvingEnabled()).isTrue();
         assertThat(config.fileCacheDir()).isNotNull();
         assertThat(config.fileCachingEnabled()).isNotNull();
     }
