@@ -36,16 +36,23 @@ import com.datastax.driver.core.LocalDate;
 import com.datastax.driver.core.utils.UUIDs;
 import io.vertx.core.Promise;
 import org.apache.cassandra.sidecar.TestModule;
+import org.apache.cassandra.sidecar.cluster.instance.InstanceMetadata;
+import org.apache.cassandra.sidecar.common.data.ConsistencyLevel;
 import org.apache.cassandra.sidecar.common.data.RestoreJobStatus;
+import org.apache.cassandra.sidecar.common.response.NodeSettings;
+import org.apache.cassandra.sidecar.common.server.cluster.locator.TokenRange;
 import org.apache.cassandra.sidecar.common.server.utils.MillisecondBoundConfiguration;
 import org.apache.cassandra.sidecar.common.server.utils.SecondBoundConfiguration;
-import org.apache.cassandra.sidecar.concurrent.ExecutorPools;
 import org.apache.cassandra.sidecar.config.RestoreJobConfiguration;
 import org.apache.cassandra.sidecar.db.RestoreJob;
 import org.apache.cassandra.sidecar.db.RestoreJobDatabaseAccessor;
+import org.apache.cassandra.sidecar.db.RestoreRange;
 import org.apache.cassandra.sidecar.db.RestoreRangeDatabaseAccessor;
 import org.apache.cassandra.sidecar.db.RestoreSliceDatabaseAccessor;
+import org.apache.cassandra.sidecar.db.RestoreSliceTest;
 import org.apache.cassandra.sidecar.db.schema.SidecarSchema;
+import org.apache.cassandra.sidecar.exceptions.CassandraUnavailableException;
+import org.apache.cassandra.sidecar.exceptions.RestoreJobExceptions;
 import org.apache.cassandra.sidecar.metrics.MetricRegistryFactory;
 import org.apache.cassandra.sidecar.metrics.SidecarMetrics;
 import org.apache.cassandra.sidecar.metrics.SidecarMetricsImpl;
@@ -53,16 +60,25 @@ import org.apache.cassandra.sidecar.tasks.PeriodicTaskExecutor;
 import org.apache.cassandra.sidecar.tasks.ScheduleDecision;
 import org.apache.cassandra.sidecar.utils.InstanceMetadataFetcher;
 import org.mockito.ArgumentCaptor;
+import org.mockito.Mockito;
 
 import static org.apache.cassandra.sidecar.db.RestoreJobTest.createNewTestingJob;
+import static org.apache.cassandra.sidecar.db.RestoreJobTest.createTestingJob;
 import static org.apache.cassandra.sidecar.db.RestoreJobTest.createUpdatedJob;
+import static org.apache.cassandra.sidecar.exceptions.CassandraUnavailableException.Service.JMX;
 import static org.apache.cassandra.sidecar.utils.TestMetricUtils.registry;
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.ArgumentMatchers.anyBoolean;
 import static org.mockito.ArgumentMatchers.anyInt;
 import static org.mockito.ArgumentMatchers.anyLong;
+import static org.mockito.ArgumentMatchers.anyShort;
 import static org.mockito.ArgumentMatchers.eq;
 import static org.mockito.Mockito.doNothing;
+import static org.mockito.Mockito.doThrow;
 import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.never;
+import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 
 class RestoreJobDiscovererTest
@@ -76,6 +92,9 @@ class RestoreJobDiscovererTest
     private final RestoreJobManagerGroup mockManagers = mock(RestoreJobManagerGroup.class);
     private final PeriodicTaskExecutor executor = mock(PeriodicTaskExecutor.class);
     private final SidecarSchema sidecarSchema = mock(SidecarSchema.class);
+    private final RingTopologyRefresher ringTopologyRefresher = mock(RingTopologyRefresher.class);
+    private final InstanceMetadataFetcher instanceMetadataFetcher = mock(InstanceMetadataFetcher.class);
+    private final NodeSettings mockNodeSettings = mock(NodeSettings.class);
     private SidecarMetrics metrics;
     private RestoreJobDiscoverer loop;
 
@@ -84,17 +103,18 @@ class RestoreJobDiscovererTest
     {
         MetricRegistryFactory mockRegistryFactory = mock(MetricRegistryFactory.class);
         when(mockRegistryFactory.getOrCreate()).thenReturn(registry());
-        InstanceMetadataFetcher mockMetadataFetcher = mock(InstanceMetadataFetcher.class);
-        metrics = new SidecarMetricsImpl(mockRegistryFactory, mockMetadataFetcher);
+        when(mockNodeSettings.datacenter()).thenReturn("dc1");
+        when(instanceMetadataFetcher.callOnFirstAvailableInstance(any())).thenReturn(mockNodeSettings);
+        metrics = new SidecarMetricsImpl(mockRegistryFactory, instanceMetadataFetcher);
         loop = new RestoreJobDiscoverer(testConfig(),
                                         sidecarSchema,
                                         mockJobAccessor,
                                         mockSliceAccessor,
                                         mockRangeAccessor,
                                         () -> mockManagers,
-                                        null,
-                                        null,
-                                        mock(ExecutorPools.class),
+                                        instanceMetadataFetcher,
+                                        ringTopologyRefresher,
+                                        executor,
                                         metrics);
     }
 
@@ -120,7 +140,6 @@ class RestoreJobDiscovererTest
                                                         .jobStatus(RestoreJobStatus.CREATED)
                                                         .expireAt(new Date(System.currentTimeMillis() + 10000L))
                                                         .build()));
-        loop.registerPeriodicTaskExecutor(executor);
         executeBlocking();
         assertThat(metrics.server().restore().activeJobs.metric.getValue()).describedAs("active jobs count is updated")
                                                                            .isOne();
@@ -163,7 +182,6 @@ class RestoreJobDiscovererTest
         ArgumentCaptor<RestoreJob> jobCapture = ArgumentCaptor.forClass(RestoreJob.class);
         doNothing().when(mockManagers).removeJobInternal(jobCapture.capture());
         when(mockJobAccessor.findAllRecent(anyLong(), anyInt())).thenReturn(mockResult);
-        loop.registerPeriodicTaskExecutor(executor);
 
         assertThat(loop.hasInflightJobs())
         .describedAs("No inflight jobs are discovery when loop has not started")
@@ -249,13 +267,61 @@ class RestoreJobDiscovererTest
         ArgumentCaptor<UUID> abortedJobs = ArgumentCaptor.forClass(UUID.class);
         doNothing().when(mockJobAccessor).abort(abortedJobs.capture(), eq("Expired"));
         when(mockJobAccessor.findAllRecent(anyLong(), anyInt())).thenReturn(mockResult);
-        loop.registerPeriodicTaskExecutor(executor);
         executeBlocking();
 
         List<UUID> expectedAbortedJobs = mockResult.stream().map(s -> s.jobId).collect(Collectors.toList());
         assertThat(abortedJobs.getAllValues()).isEqualTo(expectedAbortedJobs);
     }
 
+    @Test
+    void testDiscoverSidecarManagedJob() throws Exception
+    {
+        UUID jobId = discoverSidecarManagedJob(false);
+
+        ArgumentCaptor<RestoreRange> restoreRangeCaptor = ArgumentCaptor.forClass(RestoreRange.class);
+        verify(mockRangeAccessor).create(restoreRangeCaptor.capture());
+        assertThat(restoreRangeCaptor.getAllValues()).hasSize(1);
+        RestoreRange captured = restoreRangeCaptor.getValue();
+        assertThat(captured.jobId()).isEqualTo(jobId);
+    }
+
+    @Test
+    void testDisciverAlreadyFailedSidecarManagedJob() throws Exception
+    {
+        discoverSidecarManagedJob(true);
+        verify(mockRangeAccessor, never()).create(any());
+    }
+
+    private UUID discoverSidecarManagedJob(boolean isJobFailed) throws Exception
+    {
+        when(sidecarSchema.isInitialized()).thenReturn(true);
+        UUID jobId = UUIDs.timeBased();
+        RestoreJob sidecarManagedJob = createTestingJob(jobId, RestoreJobStatus.STAGE_READY, ConsistencyLevel.QUORUM);
+        assertThat(sidecarManagedJob.isManagedBySidecar()).isTrue();
+
+        when(mockJobAccessor.findAllRecent(anyLong(), anyInt()))
+        .thenReturn(Collections.singletonList(sidecarManagedJob));
+        when(ringTopologyRefresher.localTokenRanges(any(), anyBoolean()))
+        .thenReturn(Collections.singletonMap(1, Collections.singleton(new TokenRange(0, 100))));
+        InstanceMetadata instance = mock(InstanceMetadata.class);
+        when(instance.stagingDir()).thenReturn("stagingDir");
+        when(instanceMetadataFetcher.instance(anyInt())).thenReturn(instance);
+        when(mockSliceAccessor.selectByJobByBucketByTokenRange(any(), anyShort(), any()))
+        .thenReturn(Collections.singletonList(RestoreSliceTest.createTestingSlice(sidecarManagedJob, "sliceId", 0, 10)));
+        if (isJobFailed)
+        {
+            doThrow(RestoreJobExceptions.ofFatal("Job failed", mock(RestoreRange.class), null))
+            .when(mockManagers).trySubmit(any(), any(), any());
+        }
+        else
+        {
+            when(mockManagers.trySubmit(any(), any(), any())).thenReturn(RestoreJobProgressTracker.Status.CREATED);
+        }
+
+        executeBlocking();
+        return jobId;
+    }
+
     @Test
     void testWhenJobShouldBeLogged()
     {
@@ -298,7 +364,6 @@ class RestoreJobDiscovererTest
     {
         when(sidecarSchema.isInitialized()).thenReturn(true);
         assertThat(loop.jobDiscoveryRecencyDays()).isEqualTo(5);
-        loop.registerPeriodicTaskExecutor(executor);
 
         executeBlocking();
 
@@ -320,6 +385,59 @@ class RestoreJobDiscovererTest
         .isEqualTo(10);
     }
 
+    @Test
+    void testSkipNotOwnedRestoreToLocalDatacenterOnlyJob()
+    {
+        // Create a restore job that restores to dc2 only. Meanwhile, discoverer runs in dc1.
+        UUID jobId = UUIDs.timeBased();
+        when(mockJobAccessor.findAllRecent(anyLong(), anyInt()))
+        .thenReturn(Collections.singletonList(RestoreJob.builder()
+                                                        .createdAt(RestoreJob.toLocalDate(jobId))
+                                                        .jobId(jobId)
+                                                        .jobAgent("agent")
+                                                        .jobStatus(RestoreJobStatus.CREATED)
+                                                        .expireAt(new Date(System.currentTimeMillis() + 10000L))
+                                                        .consistencyLevel(ConsistencyLevel.LOCAL_QUORUM)
+                                                        .localDatacenter("dc2")
+                                                        .shouldRestoreToLocalDatacenterOnly(true)
+                                                        .build()));
+        executeBlocking();
+        verify(mockRangeAccessor, never()).create(any());
+    }
+
+    @Test
+    void testRestoreToLocalDatacenterOnlyJobIsOnHoldWhenLocalDatacenterIsUndetermined()
+    {
+        // local datacenter is undetermined and the restore job is configured to restore to local datacenter only.
+        // the job is on hold until local datacenter is resolved in discoverer.
+        when(instanceMetadataFetcher.callOnFirstAvailableInstance(any())).thenThrow(new CassandraUnavailableException(JMX, "NodeSettings unavailable"));
+        UUID jobId = UUIDs.timeBased();
+        when(mockJobAccessor.findAllRecent(anyLong(), anyInt()))
+        .thenReturn(Collections.singletonList(RestoreJob.builder()
+                                                        .createdAt(RestoreJob.toLocalDate(jobId))
+                                                        .jobId(jobId)
+                                                        .jobAgent("agent")
+                                                        .jobStatus(RestoreJobStatus.CREATED)
+                                                        .expireAt(new Date(System.currentTimeMillis() + 10000L))
+                                                        .consistencyLevel(ConsistencyLevel.LOCAL_QUORUM)
+                                                        .localDatacenter("dc1")
+                                                        .shouldRestoreToLocalDatacenterOnly(true)
+                                                        .build()));
+
+        executeBlocking();
+        verify(ringTopologyRefresher, never()).register(any(), any());
+
+        // in the new run, the local datacenter is discovered. The discoverer proceeds further and register the job
+        Mockito.reset(instanceMetadataFetcher);
+        when(instanceMetadataFetcher.callOnFirstAvailableInstance(any())).thenReturn(mockNodeSettings);
+        executeBlocking();
+        ArgumentCaptor<RestoreJob> restoreJobCaptor = ArgumentCaptor.forClass(RestoreJob.class);
+        verify(ringTopologyRefresher).register(restoreJobCaptor.capture(), any());
+        assertThat(restoreJobCaptor.getAllValues()).hasSize(1);
+        RestoreJob captured = restoreJobCaptor.getValue();
+        assertThat(captured.jobId).isEqualTo(jobId);
+    }
+
     private RestoreJobConfiguration testConfig()
     {
         RestoreJobConfiguration restoreJobConfiguration = mock(RestoreJobConfiguration.class);
