@@ -298,6 +298,7 @@ class RestoreRangeTaskTest
         RestoreJob job = spy(RestoreJobTest.createTestingJob(UUIDs.timeBased(), RestoreJobStatus.IMPORT_READY));
         doReturn(true).when(job).isManagedBySidecar();
         doReturn(job).when(mockRange).job();
+        doReturn(true).when(mockRange).hasStaged();
         RestoreRangeTask task = createTask(mockRange, job);
 
         Promise<RestoreRange> promise = Promise.promise();
@@ -310,6 +311,35 @@ class RestoreRangeTaskTest
         assertThat(mockRange.hasImported()).isTrue();
     }
 
+    @Test
+    void testSliceNotStagedInImportPhase()
+    {
+        // test specific setup
+        RestoreJob job = spy(RestoreJobTest.createTestingJob(UUIDs.timeBased(), RestoreJobStatus.IMPORT_READY));
+        doReturn(true).when(job).isManagedBySidecar();
+        doReturn(job).when(mockRange).job();
+        doReturn(false).when(mockRange).hasStaged();
+        doReturn(Paths.get("nonexist")).when(mockRange).stagedObjectPath();
+        HeadObjectResponse headObjectResponse = mock(HeadObjectResponse.class);
+        when(headObjectResponse.contentLength()).thenReturn(1L);
+        when(mockStorageClient.objectExists(mockRange)).thenReturn(CompletableFuture.completedFuture(headObjectResponse));
+        when(mockStorageClient.downloadObjectIfAbsent(eq(mockRange), any(TaskExecutorPool.class)))
+        .thenReturn(Future.succeededFuture(new File(".")));
+        RestoreRangeTask task = createTask(mockRange, job);
+
+        Promise<RestoreRange> promise = Promise.promise();
+        task.handle(promise);
+        getBlocking(promise.future()); // no error is thrown
+
+        verify(mockRange, times(0)).completeStagePhase(); // should not be called in the phase
+        verify(mockRange, times(1)).completeImportPhase();
+        verify(mockRangeAccessor, times(1)).updateStatus(mockRange);
+        // verify that the task should download object from storage service
+        verify(mockStorageClient, times(1)).objectExists(mockRange);
+        verify(mockStorageClient, times(1)).downloadObjectIfAbsent(eq(mockRange), any(TaskExecutorPool.class));
+        assertThat(mockRange.hasImported()).isTrue();
+    }
+
     @Test
     void testImportSSTables(@TempDir Path testFolder)
     {
@@ -377,6 +407,7 @@ class RestoreRangeTaskTest
         RestoreJob job = RestoreJobTest.createTestingJob(UUIDs.timeBased(), RestoreJobStatus.IMPORT_READY, ConsistencyLevel.QUORUM);
         Path stagedPath = testFolder.resolve("slice.zip");
         Files.createFile(stagedPath);
+        when(mockRange.hasStaged()).thenReturn(true);
         when(mockRange.stagedObjectPath()).thenReturn(stagedPath);
         Promise<RestoreRange> promise = Promise.promise();
         RestoreRangeTask task = createTaskWithExceptions(mockRange, job);
@@ -384,7 +415,7 @@ class RestoreRangeTaskTest
 
         assertThatThrownBy(() -> getBlocking(promise.future()))
         .satisfies(throwable -> assertThat(ThrowableUtils.getCause(throwable, RestoreJobException.class))
-                                .hasCauseInstanceOf(RuntimeException.class)
+                                .hasRootCauseExactlyInstanceOf(RuntimeException.class)
                                 .hasMessage("Random exception"));
     }
 
